/*
$RCSfile: tracer13.c,v $
$Id: tracer13.c,v 1.1 2001/04/17 21:34:10 pgtc Exp pgtc $
$Log: tracer13.c,v $
Revision 1.1  2001/04/17 21:34:10  pgtc
Initial revision

Tracer 13.c
Same as all the previous CAESAR models, e.g swale7 etc.. but has a tracer element added
This should become the generic version, with the tracer element being removed by setting 
number of tracer layers to 1, and removing/comenting out the adding lead section.

Improvements over previous versions include addition of a simple lateral erosion model

*/

#include "stdio.h"
#include "math.h"

/*** reach=lenght, xsection=width, active=active layer thickness **/

#define reach 204
#define xsection 887
#define ACTIVE_FACTOR 1
#define TRUE 1
#define FALSE 0
#define ERODEFACTOR 0.05 /* normally 0.02 */
#define DX 50
#define root 70.71  /* must be equal to root (2 x DX) */
#define LIMIT 3 
#define MIN_Q 0.5 /* must be 0.001 * pow(DX,2) , since discovered 0.002 better..*/
#define CREEP_RATE 0.0025

/** these all save me writing the code in every time **/

#define	N index[x][y-1]
#define	NE index[x+1][y-1]
#define	E index[x+1][y]
#define	SE index[x+1][y+1]
#define	S index[x][y+1]
#define	SW index[x-1][y+1]
#define	W index[x-1][y]
#define	NW index[x-1][y-1]
#define CENTRE index[x][y]
#define CENTRE2 index[x2][y2]
#define active 0.2  /* depth of active layer */
#define G_MAX 10    /** number of grainsize fractions */
#define TRACERS 1  /* number of fraction types */

#define lateral_constant 0.000002 /*normally 0.000001 - 1m a month */

/************* globals *****************/


static float bedrock[xsection+10][reach+6];
static float init_elevs[xsection+10][reach+6];

static float max_flow[xsection+10][reach+6][9]; /* flow out values for cell 1-8 1 being north */
static int flow_dir[xsection+10][reach+6][2]; /* 0 contains max flow in direction, 1 contains max flow out direction */

static float strata [(reach*xsection)/LIMIT][10][G_MAX-1][TRACERS];
static float grain [(reach*xsection)/LIMIT][G_MAX+1][TRACERS];   /* holds grainsize values   */
static double temp [(reach*xsection)/LIMIT][G_MAX+1][TRACERS];    /* holds temp grainsizes    */
static double dist [9][G_MAX+1];			/* distribution for erosion routine */
static double extra_elev[(reach*xsection)/LIMIT];

static double elev[xsection+10][reach+6];
static float discharge[xsection+10][reach+6];
static float water_depth[xsection+10][reach+6];
static float area[xsection+10][reach+6];
static int index[xsection+10][reach+6];



static int cross_scan[xsection+1][reach+3];	/* for cross scanssss...  */
static int down_scan[reach+1][xsection+3];     /* for side scans     */
static float hourly_rain_data[100000];		/* array where we stick the hourly raindata */
static float bedrock_depth[reach+5];		/* contains bedrock depth for each x section. */
static float climate_data[10000][3];
static int grain_array_tot = 0;
static int minesites[1000][2];
static float temp_grain[G_MAX+1];


static int y_min=2;
static int init_flag=0;
static float time_factor;	/* main time step variable */
static float j=0.000000001;	/* topmodel parameters */
static float jo=0.000000001;	/* topmodel parameters */
static float j_mean;		/* topmodel parameters */
static float old_j_mean=0;
static float new_j_mean=0;
static float M=0.0145;	 	/* topmodel parameters */
static float K=0.01;	  	/* topmodel parameters  */
static float input=0;     	/* rainfall in m per m square per timestetp (here secs) */
static float rain_fall_rate=0.00001;  	/* in m/sec */
static float baseflow=0.0000005; /* basic normal discharge */
static double cycle=0;
static float rain_factor=1;
static int year;
static int year_inc;
static float eroded=0;
static float deposited=0;
/**************************************************************/
 
	/* function declarations */
int inter();
void add_lead();
void calc_flow_direction();
void lateral();
void save_data50();
void slide();
void save_data();
void save_data2();
void save_data3();
void erode2();
void erode3();
void find_active();
void hydrograph();
void save_all_data();
void addGS();
float four_dp();
void init_route();
void load_data();
void load_all_data();
void zero_values();
void route_down();
void route_up();
float route_lr();
void route_rl();
void erode();
void slide_GS();
void sort_active();
void calc_J();
void calc_area_and_slope();
void get_area();
void get_area2();
void slide_2();
void slide_3();
void creep();
void grow_grass();
void erode4();
float grain_tot();
float temp_tot();
float strata_tot();


/*********************** start of main ***************************/

main()
{
  
  /********* locals ***********/
  int x,y,n,p;
  int first_init_flag=0;
  double time_1;
  double time_2;
  float inca=10;
  float incb=100;
  float iter=0;
  float incc=259200;
  float incz=100;
  float totty;
  float tag=0;
  float tempflow=baseflow;
  float incd=0; /* time delay for creep, a month. 40300 */
  float ince=cycle+60;
  

  
  /**************** initalise/load data *******/
  
  time_1=1;
  
  zero_values();
  
  printf("Zero'd\n");


  time_factor=60;

  /*	load_data(); *//* remmed out but kept in for making new master file... */
  calc_J(1.0);
  printf("done3\n");
  
  printf("loading\n");
  
  load_all_data();
  /*load_data(); */
  
  /*save_data2(1.0);*/ /*remmed out as used to print grainsize */
  
  time_factor=1;
  
  /**************** main loop *****************/
  /*    	slide(1);
		
		slide(1);
  */
  /**	save_all_data(cycle);
	remmed out but kept in for making new master file...
	printf(" definitely kill now \n"); **/
  
  get_area();
  
  printf(" kill now \n");
  
  init_route(TRUE,tempflow);
  
  printf(" about to erode! \n");
  
  erode();
  
  init_flag=0;
  
  printf(" sliding !! \n");	 
  /*
    slide(1);
  */
  printf(" eroded starting main loop \n");
  
  /*	calc_area_and_slope();*/
  /**************************************loop******/
  
  for(year=1700;year<=2450;year+=50){
    save_data50(year);
    save_data2(year);
    
    M=0.02;
    rain_factor=1.5;
    
    if(year<=9250){
      M=climate_data[year][1];
      rain_factor=climate_data[year][2];
    }
    if(year>8000&&M<0.02)M=0.02;
    creep(50.0);
    
    
    for(year_inc=1;year_inc<=5;year_inc++){
      /*	cycle=0;*/
      save_data50(year+((year_inc-1)*10));
      if(cycle>5200000)cycle=10;
      inca=10;
      incb=cycle+100;
      iter=0;
      incc=cycle+1440;
      time_1=1;
      
      ince=cycle+4030;
      incd=cycle+1000000; /* about 2 years!!*/
      incz=0;
      
      do{
	
	
	/****** hydrograph shit *********/
	/** updating hydrograph in 2 min chumks **/
	
	if(cycle>=time_1){
	  do{
	    time_1+=2;
	    calc_J(time_1);
	    if(time_factor>3600&&new_j_mean>(0.2/3000000)){
	      cycle=time_1+60;
	      time_factor=3600;
	    }		   
	  }while(time_1<cycle);
	}
	
	hydrograph(time_1-cycle);
	
	/*j_mean=baseflow*0.00009;*/
	
	
	/************************  water routing ***************************/
	
	route_up();
	route_down(cycle);	
	route_rl();
	route_lr();
	
	calc_flow_direction();
	
	/********************** checks to see if area needs redefining ********************************/
	
	if(inca<=iter){	
	  get_area();
	  init_route(TRUE,tempflow);
	  inca+=1000;
	  save_all_data(cycle);
	  
	  /*	save_data2(cycle);*/
	}
	
	if(j_mean>=tempflow){
	  /*        save_data2(cycle);*/
	  tempflow=tempflow*1.5;
	  get_area();
	  init_route(TRUE,tempflow);
	  printf("aaa\n");
	  time_factor=0.0000000001;
	}
	
	
	if(tempflow>(j_mean*1.5)){
	  tempflow=j_mean*1.01;
	  if(tempflow>baseflow){
	    get_area();
	    init_route(TRUE,tempflow);
	    printf("baa\n");
	    time_factor=0.0000000001;
	  }
	  else{
	    tempflow=baseflow;
	  }
	}
	
	
	/************************** saves data ****************************/

	/*** adding lead every day...*/
#if 0
	if(cycle>=incc){
	  do{
	    /*printf("adding lead!\n");*/
	    add_lead();
	    incc+=1440;
	  }while(cycle>incc);
	}
#endif

	/** grass growing, before erode, as uses water... */
	
	
	if(cycle>=ince){
	  ince=cycle+4032;
	  /* call every 40 hours..(changed from 10 !!) */
	  grow_grass();
	  /* therfore growing grass in 8 years..*/
	  /* in this version every 0.8 years!*/
	  
	}
	
	/* saving data bits..*/
	
	if(iter>=incz){
	  incz+=1000;
	  printf("saving now!!\n\n\n");
	  save_data2(incz);
	  save_data3(incz);
	}
	
	/*********************** eroding & sliding *************************/
	
	
	if(time_factor>180000)time_factor=180000;
	if(j_mean>(0.2/3000000)&&time_factor>1800)time_factor=1800;
	
	
	erode(); 
	
	lateral();
	
	/************************8 do Landslides. **************************/
	/** if you want them to recognise we/dry slides, place before erode routine. */
	
	
	if(incb<=iter){
	  slide(1);
	  incb+=1000;
	}

	slide_3();
	
	
	if(incd<=cycle){
	  creep(2.0);
	  incd+=2*525600;
	}
	
	
	
	/********************************************************************/
	
	cycle+=(time_factor/60);
	iter++;
	/** rainfallrate now redundant **/
	/** if(cycle>3600)rain_fall_rate=baseflow; **/
	
	/*********** cycle now in minuites !!!!! ****************/
	
      }while(cycle<=5256000);  /** 5256000 */

      save_all_data(cycle);
      
    }/* end of 50 year loop */
    
    
  }/* end of 150 year loop */   
}


/************************ end of main *****************************/
void add_lead()
{
  int x,y,z,inc;
  float value;

  /** just adds a set level to a certain range of grid cells.. nothing fancy 
   now addind daily to certain cells only...*/
  value=0;
  if(year==1700||year==1750)value=0.0001;/*0.00002739;*/
  if(year==1800||year==1850)value=0.0002;/*0.0000548;*/

  inc=0;
  do{
    x=minesites[inc][0];
    y=minesites[inc][1];
    if(index[x][y]==-9999)addGS(x,y);
    grain[CENTRE][1][1]+=value/2;
    grain[CENTRE][2][1]+=value/2;
    elev[x][y]+=value;
    sort_active(x,y);
    inc++;

  }while(minesites[inc][0]>0);

}

/*****************************************************************/



int inter(infloat)
float infloat;
{
  int n;
  n=infloat;
  return(n);
}

/********************* Shady little hydrograph routine ************/

void hydrograph(time)
float time;
{


	j_mean=old_j_mean+(((new_j_mean-old_j_mean)/2)*(2-time));		

}

/*************************** calc J's ******************************/


void calc_J(cycle)
float cycle;
{

	float local_rain_fall_rate; /** in m/second **/
	float local_time_step=120; /** in secs */
	old_j_mean=new_j_mean;
	jo=j;

	local_rain_fall_rate=0;
	if(hourly_rain_data[(int)(cycle/60)]>0){
		local_rain_fall_rate=rain_factor*((hourly_rain_data[(int)(cycle/60)]/1000)/3600); /** divide by 1000 to make m/hr, then by 3600 for m/sec */
	}

	if (local_rain_fall_rate==0){
		j=jo/(1+((jo*local_time_step)/M));
		new_j_mean=M/local_time_step*log(1+((jo*local_time_step)/M));		
	}

	if (local_rain_fall_rate>0){
		j=local_rain_fall_rate/(((local_rain_fall_rate-jo)/jo)*exp((0-local_rain_fall_rate)*local_time_step/M)+1);
		new_j_mean=(M/local_time_step)*log(((local_rain_fall_rate-jo)+jo*exp((local_rain_fall_rate*local_time_step)/M))/local_rain_fall_rate);
	}
	if(new_j_mean<0)new_j_mean=0;
	/**printf("new J mean = %f\n",(new_j_mean*1000));*/ 

}


/************************** calc area & min slope *****************/

void calc_area_and_slope()
{

}

/******************** grow grass ***********************************/

void grow_grass()
{

  /****** this has to have its own version of sort active not removing grass ***/


  int x,y,n,z,temp_centre,q,p;
  float total,total2,sub_tot;
  float amount3=0.000961;
  float amount2;
  float amount=0;

  for(x=1;x<=xsection;x++){
    for(y=1;y<=reach;y++){
      if(index[x][y]!=-9999&&water_depth[x][y]<=0){

	/** 0.000057 is adding grass to 0.1 at ten hourly steps
	  so its 0.01 from 0 in 2 years **/

	if(grain[CENTRE][0][0]<=(active-amount3)&&grain[CENTRE][G_MAX][0]<=(active-amount3)){
	  sort_active(x,y);

	  grain[CENTRE][0][0]+=amount3;
 	  
	  total=0;
	  temp_centre=CENTRE;
	  /** has to run from n=1 */

	  for(n=0;n<=G_MAX;n++){
		if(grain[CENTRE][n][0]<0){
			printf("SHITE  1 !! %f %d %d \n",grain[CENTRE][n][0],x,y);
			grain[CENTRE][n][0]=0;
		}
		total+=grain_tot(CENTRE,n);
	  }

	  amount=total-active;
	  if(amount<0)printf("asdasd %f\n",(amount*1000000));

		if(grain[temp_centre][G_MAX][0]>0){
		  if(grain[temp_centre][G_MAX][0]>=amount){
		   grain[temp_centre][G_MAX][0]-=amount;
		   amount=0;
		  }
		  else{
		   amount-=grain[temp_centre][G_MAX][0];
		   grain[temp_centre][G_MAX][0]=0;
		  }
		}

		for(n=1;n<=(G_MAX-1);n++){
			if(grain_tot(temp_centre,n)>0){
				amount2=(grain_tot(temp_centre,n)/total)*amount;
			    	if(n>0&&n<G_MAX){
				  sub_tot=grain_tot(temp_centre,n);
				  if(sub_tot>0){
	      	      		   for(q=0;q<=(TRACERS-1);q++){
				     strata[temp_centre][0][n-1][q]+=amount2*(grain[temp_centre][n][q]/sub_tot);
				   }
				  }
				}
				sub_tot=grain_tot(temp_centre,n);
				if(sub_tot>0){
				  for(q=0;q<=(TRACERS-1);q++){
				    grain[temp_centre][n][q]-=amount2*(grain[temp_centre][n][q]/sub_tot);
				  }
				}
			}
		}
		       	
			/* then workk down through the strata **/

		for(z=0;z<=9;z++){
			total2=0;
			for(n=0;n<=(G_MAX-2);n++)total2+=strata_tot(temp_centre,z,n);
			amount=(total2-(active));
			for(n=0;n<=(G_MAX-2);n++){
				if(strata_tot(temp_centre,z,n)>0&&amount>0){
				  if(z<9){
				    sub_tot=strata_tot(temp_centre,z,n);
				    for(q=0;q<=(TRACERS-1);q++){
				      strata[temp_centre][z+1][n][q]+=(strata[temp_centre][z][n][q]/sub_tot)*((sub_tot/total2)*amount);
				    }
				  }
				  sub_tot=strata_tot(temp_centre,z,n);
				  for(q=0;q<=(TRACERS-1);q++){
					strata[temp_centre][z][n][q]-=(strata[temp_centre][z][n][q]/sub_tot)*((sub_tot/total2)*amount);
				  }
				}	
			}		
		}
  
		

	  
	  sort_active(x,y);
	}
      }
    }
  }

}


/************************ creep **************************************/


void creep(time)
float time;
{

	/** creep rate is 10*-2 * slope per year, so inputs time jump in years*/
	/** very important differnece here is that slide_GS() is called only if
		BOTH cells are not -9999 therfore if both have grainsize then do additions.
		this is to stop the progressive spread of selected cells upslope */



	int x,y;
	float temp,amount;
	
	for(x=1;x<=xsection;x++){
		for(y=1;y<=reach;y++){
			discharge[x][y]=100;
		}
	}

	
	for(x=2;x<xsection;x++){
		for(y=2;y<reach;y++){
			if(elev[x][y-1]<elev[x][y]&&elev[x][y-1]>0){
				temp=((elev[x][y]-elev[x][y-1])/DX)*CREEP_RATE*time/DX;
				if((elev[x][y]-temp)<bedrock[x][y])temp=elev[x][y]-bedrock[x][y];
				if(temp<0)temp=0;
				
				discharge[x][y]-=temp;
				discharge[x][y-1]+=temp;
			}
			if(elev[x+1][y-1]<elev[x][y]&&elev[x+1][y-1]>0){
				temp=((elev[x][y]-elev[x+1][y-1])/root)*CREEP_RATE*time/DX;
				if((elev[x][y]-temp)<bedrock[x][y])temp=elev[x][y]-bedrock[x][y];
				if(temp<0)temp=0;
				
				discharge[x][y]-=temp;
				discharge[x+1][y-1]+=temp;
			
			}
			if(elev[x+1][y]<elev[x][y]&&elev[x+1][y]>0){
				temp=((elev[x][y]-elev[x+1][y])/DX)*CREEP_RATE*time/DX;
				if((elev[x][y]-temp)<bedrock[x][y])temp=elev[x][y]-bedrock[x][y];
				if(temp<0)temp=0;
				
				discharge[x][y]-=temp;
				discharge[x+1][y]+=temp;
			
			}
			if(elev[x+1][y+1]<elev[x][y]&&elev[x+1][y+1]>0){
				temp=((elev[x][y]-elev[x+1][y+1])/root)*CREEP_RATE*time/DX;
				if((elev[x][y]-temp)<bedrock[x][y])temp=elev[x][y]-bedrock[x][y];
				if(temp<0)temp=0;
				
				discharge[x][y]-=temp;
				discharge[x+1][y+1]+=temp;
			
			}
			if(elev[x][y+1]<elev[x][y]&&elev[x][y+1]>0){
				temp=((elev[x][y]-elev[x][y+1])/DX)*CREEP_RATE*time/DX;
				if((elev[x][y]-temp)<bedrock[x][y])temp=elev[x][y]-bedrock[x][y];
				if(temp<0)temp=0;
				
				discharge[x][y]-=temp;
				discharge[x][y+1]+=temp;
			
			}
			if(elev[x-1][y+1]<elev[x][y]&&elev[x-1][y+1]>0){
				temp=((elev[x][y]-elev[x-1][y+1])/root)*CREEP_RATE*time/DX;
				if((elev[x][y]-temp)<bedrock[x][y])temp=elev[x][y]-bedrock[x][y];
				if(temp<0)temp=0;
				
				discharge[x][y]-=temp;
				discharge[x-1][y+1]+=temp;
			
			}
			if(elev[x-1][y]<elev[x][y]&&elev[x-1][y]>0){
				temp=((elev[x][y]-elev[x-1][y])/DX)*CREEP_RATE*time/DX;
				if((elev[x][y]-temp)<bedrock[x][y])temp=elev[x][y]-bedrock[x][y];
				if(temp<0)temp=0;
		     
				discharge[x][y]-=temp;
				discharge[x-1][y]+=temp;
			
			}
			if(elev[x-1][y-1]<elev[x][y]&&elev[x-1][y-1]>0){
				temp=((elev[x][y]-elev[x-1][y-1])/root)*CREEP_RATE*time/DX;
				if((elev[x][y]-temp)<bedrock[x][y])temp=elev[x][y]-bedrock[x][y];
				if(temp<0)temp=0;

				discharge[x][y]-=temp;
				discharge[x-1][y-1]+=temp;
			
			}
		}
	}
	
	for(x=1;x<=xsection;x++){
		for(y=1;y<=reach;y++){
		  discharge[x][y]-=100;
		  elev[x][y]+=discharge[x][y];
		  
			/** next bit for adding material to channel from creep **/
		  	if(index[x][y]!=-9999&&discharge[x][y]>0){
			  	if(grain[CENTRE][G_MAX][0]>0){
				  amount=discharge[x][y];
				  if(grain[CENTRE][G_MAX][0]>=amount){
				    grain[CENTRE][G_MAX][0]-=amount;
				    amount=0;
				  }
				  else{
				    amount-=grain[CENTRE][G_MAX][0];
				    grain[CENTRE][G_MAX][0]=0;
				  }
				  sort_active(x,y);
				}
			}

			discharge[x][y]=0;
		}
	}

}



/************************** init routing. *************************/

void get_area()
{
  int x,y,n;

  for(x=1;x<=xsection;x++){
    for(y=1;y<=reach;y++){
                        water_depth[x][y]=1;
			area[x][y]=0;
    }
  }
  get_area2();
  
}



void get_area2()
  {		
	
	int x,y,n,inc;
	float factor,ht, tot_q;
	float q_out;
	float temp_a;
	y_min=1;

/****** route lr *****/
	

	for(x=1;x<xsection;x++){
		for(y=1;y<reach;y++){
		
			ht=elev[x][y];

			if(elev[x+1][y-1]<=ht||elev[x+1][y]<=ht||elev[x+1][y+1]<=ht){

				ht+=0.00001; 

				temp_a=0;
				tot_q=0;
				q_out=water_depth[x][y];
				if(water_depth[x][y]>area[x][y])area[x][y]=water_depth[x][y];


				if(elev[x+1][y-1]<ht)
					tot_q+=((ht-elev[x+1][y-1])/root);
				if(elev[x+1][y]<ht)
					tot_q+=(ht-elev[x+1][y])/DX;
				if(elev[x+1][y+1]<ht)
					tot_q+=((ht-elev[x+1][y+1])/root);

				if(elev[x+1][y-1]<ht){
					water_depth[x+1][y-1]+=q_out*(((ht-elev[x+1][y-1])/root)/tot_q);
					temp_a+=q_out*(((ht-elev[x+1][y-1])/root)/tot_q);
				}

				if(elev[x+1][y]<ht){
					water_depth[x+1][y]+=q_out*(((ht-elev[x+1][y])/DX)/tot_q);
					temp_a+=q_out*(((ht-elev[x+1][y])/DX)/tot_q);
				}

				if(elev[x+1][y+1]<ht){


					water_depth[x+1][y+1]+=q_out*(((ht-elev[x+1][y+1])/root)/tot_q);
					temp_a+=q_out*(((ht-elev[x+1][y+1])/root)/tot_q);
				}
				water_depth[x][y]-=temp_a;
	
			}
		}
	}
			


/****** route rl *******/

	for(x=xsection;x>1;x--){
		for(y=1;y<reach;y++){

	
			ht=elev[x][y];

			if(elev[x-1][y-1]<=ht||elev[x-1][y]<=ht||elev[x-1][y+1]<=ht){

				ht+=0.00001;

			        temp_a=0;
				tot_q=0;
				q_out=water_depth[x][y];
				if(water_depth[x][y]>area[x][y])area[x][y]=water_depth[x][y];



				if(elev[x-1][y-1]<ht)
					tot_q+=((ht-elev[x-1][y-1])/root);
				if(elev[x-1][y]<ht)
					tot_q+=(ht-elev[x-1][y])/DX;
				if(elev[x-1][y+1]<ht)
					tot_q+=((ht-elev[x-1][y+1])/root);

				if(elev[x-1][y-1]<ht){
					water_depth[x-1][y-1]+=q_out*(((ht-elev[x-1][y-1])/root)/tot_q);
					temp_a+=q_out*(((ht-elev[x-1][y-1])/root)/tot_q);
				}

				if(elev[x-1][y]<ht){
					water_depth[x-1][y]+=q_out*(((ht-elev[x-1][y])/DX)/tot_q);
					temp_a+=q_out*(((ht-elev[x-1][y])/DX)/tot_q);
				}

				if(elev[x-1][y+1]<ht){
					water_depth[x-1][y+1]+=q_out*(((ht-elev[x-1][y+1])/root)/tot_q);
					temp_a+=q_out*(((ht-elev[x-1][y+1])/root)/tot_q); 
				}
				water_depth[x][y]-=temp_a;
			}
		}
	}
		
	


/*   route up  ********/

	for(y=reach;y>1;y--){
		for(x=1;x<xsection;x++){


			ht=elev[x][y];

			if(elev[x-1][y-1]<=ht||elev[x][y-1]<=ht||elev[x+1][y-1]<=ht){
				
				ht+=0.00001;

				tot_q=0;
				temp_a=0;

				q_out=water_depth[x][y];
				if(water_depth[x][y]>area[x][y])area[x][y]=water_depth[x][y];


				if(elev[x-1][y-1]<ht)
					tot_q+=((ht-elev[x-1][y-1])/root);
				if(elev[x][y-1]<ht)
					tot_q+=(ht-elev[x][y-1])/DX;
				if(elev[x+1][y-1]<ht)
					tot_q+=((ht-elev[x+1][y-1])/root);

				if(elev[x-1][y-1]<ht){
					water_depth[x-1][y-1]+=q_out*(((ht-elev[x-1][y-1])/root)/tot_q);
					temp_a+=q_out*(((ht-elev[x-1][y-1])/root)/tot_q);
				}

				if(elev[x][y-1]<ht){
					water_depth[x][y-1]+=q_out*(((ht-elev[x][y-1])/DX)/tot_q);
					temp_a+=q_out*(((ht-elev[x][y-1])/DX)/tot_q);
				}

				if(elev[x+1][y-1]<ht){
					water_depth[x+1][y-1]+=q_out*(((ht-elev[x+1][y-1])/root)/tot_q);
					temp_a+=q_out*(((ht-elev[x+1][y-1])/root)/tot_q);
				}

				water_depth[x][y]-=temp_a;

			}
			
		}

	}


/*   route down  ********/

	for(y=1;y<reach;y++){
		for(x=1;x<xsection;x++){


			ht=elev[x][y];

			if(elev[x-1][y+1]<=ht||elev[x][y+1]<=ht||elev[x+1][y+1]<=ht){
				
				ht+=0.00001;

				tot_q=0;
				temp_a=0;

				q_out=water_depth[x][y];
				if(water_depth[x][y]>area[x][y])area[x][y]=water_depth[x][y];


				if(elev[x-1][y+1]<ht)
					tot_q+=((ht-elev[x-1][y+1])/root);
				if(elev[x][y+1]<ht)
					tot_q+=(ht-elev[x][y+1])/DX;
				if(elev[x+1][y+1]<ht)
					tot_q+=((ht-elev[x+1][y+1])/root);

				if(elev[x-1][y+1]<ht){
					water_depth[x-1][y+1]+=q_out*(((ht-elev[x-1][y+1])/root)/tot_q);
					temp_a+=q_out*(((ht-elev[x-1][y+1])/root)/tot_q);
				}

				if(elev[x][y+1]<ht){
					water_depth[x][y+1]+=q_out*(((ht-elev[x][y+1])/DX)/tot_q);
					temp_a+=q_out*(((ht-elev[x][y+1])/DX)/tot_q);
				}

				if(elev[x+1][y+1]<ht){
					water_depth[x+1][y+1]+=q_out*(((ht-elev[x+1][y+1])/root)/tot_q);
					temp_a+=q_out*(((ht-elev[x+1][y+1])/root)/tot_q);
				}

				water_depth[x][y]-=temp_a;

			}
			
		}

	}

}



/****************** init route, delineates area arount wet cells ********************/
/******************* flag if true is to do routing as well ie init. *****************/




void init_route(flag,input_amount)
int flag;
float input_amount;
{
	int x,y,n,inc, tot1,dan,loop;
	float slope,limit,factor,ht, tot_q, temp;
	float q_out,temp_j_mean,nowt;
	float temp_a;
	float w = 0.001;

	y_min=1;
	/* w is the water threshold, above which it recognises etc */

	temp_j_mean=j_mean;

/****** 0 mins and max's ******/


	if(flag==TRUE){
		for(x=1;x<=xsection;x++){
			for(y=1;y<=reach;y++){
			  	discharge[x][y]+=input_amount*pow(DX,2);
		 	 	water_depth[x][y]=0;
				cross_scan[x][y]=y;
				down_scan[y][x]=x;
				for(n=0;n<=8;n++)max_flow[x][y][n]=0;
				for(n=0;n<=1;n++)flow_dir[x][y][n]=0;
			}
		}
	}


	printf("\n");
	j_mean=input_amount;

       	if(flag==TRUE){

		route_lr();
		route_rl(); 
	      	route_up();
  	     	route_down();
		printf("\n");
		loop=0;
		for(dan=1;dan<=25;dan++){
		for(x=1;x<=xsection;x++){
			for(y=1;y<=reach;y++){
				water_depth[x][y]=0;
				discharge[x][y]+=input_amount*pow(DX,2);
			}
		}
		nowt=0;

	      	route_up();
  	     	route_down();
		route_rl();
		nowt=route_lr();

		printf("\n");
		if(nowt<150&&dan>=20)dan--;
		loop++;
		if(loop>=100)dan=25;
		}
	}


	/*******************************/
	
	for(x=1;x<=xsection;x++){
		for(y=1;y<=reach;y++){
			cross_scan[x][y]=0;
			down_scan[y][x]=0;
			if(water_depth[x][y]==-9999)water_depth[x][y]=0;
		}
	}
			
	for(x=1;(x<=xsection);x++){
		for(y=1;y<=(reach);y++){
		  
		  	if(water_depth[x][y]<w&&water_depth[x+1][y]<w&&water_depth[x+2][y]<w&&water_depth[x-1][y]<w&&water_depth[x-2][y]<w
				&&water_depth[x+3][y]<w&&water_depth[x-3][y]<w&&water_depth[x][y-1]<w
			   	
			   &&water_depth[x][y+1]<w&&water_depth[x][y-2]<w&&water_depth[x][y+2]<w&&water_depth[x][y+3]<w
			   &&water_depth[x][y-3]<w){ 
				water_depth[x][y]=-9999;
			} 
			if(elev[x][y]==-9999)water_depth[x][y]=-9999;
		}
	}


	for(x=1;x<=xsection;x++){
		inc=1;
		for(y=1;y<=reach;y++){
			if(water_depth[x][y]>-9999){
				cross_scan[x][inc]=y;
				inc++;
			}	
			discharge[x][y]=0;			
		}
	}	

	for(y=1;y<=reach;y++){
		inc=1;
		for(x=xsection;x>=1;x--){
			if(water_depth[x][y]>-9999){
				down_scan[y][inc]=x;
				inc++;
			}
			discharge[x][y]=0;
		}
	}
	
	for(y=1;y<=reach;y++){

		inc=1;
		while(down_scan[y][inc]>0){
		x=down_scan[y][inc];
		inc++;	
			water_depth[x][y]=0;
			discharge[x][y]=0;
		}
	}
	j_mean=temp_j_mean;
}



/**** zero_values() - zeros array **************/

void zero_values()
{
	int x,y,z,n;
	float act=0.5;
	for(y=1;y<=reach;y++){
		for(x=1;x<=xsection;x++){
			area[x][y]=0;
			elev[x][y]=0;
			bedrock[x][y]=0;
			init_elevs[x][y]=elev[x][y];
			discharge[x][y]=input;
			water_depth[x][y]=0;
			index[x][y]=-9999;

			for(n=0;n<=8;n++)max_flow[x][y][n]=0;
			for(n=0;n<=1;n++)flow_dir[x][y][n]=0;
		}
	}

	for(x=1;x<=((xsection*reach)/LIMIT);x++){
		for(y=0;y<=G_MAX;y++){
		  for(n=0;n<=(TRACERS-1);n++){
			grain[x][y][n]=0;
			temp[x][y][n]=0;
			extra_elev[x]=0;
		  }
		}
		for(z=0;z<=9;z++){
			for(y=0;y<=G_MAX-2;y++){
			  for(n=0;n<=(TRACERS-1);n++){
				strata[x][z][y][n]=0;
			  }
			}
		}
	}
	for(x=0;x<=1000;x++){
	  minesites[x][0]=0;
	  minesites[x][1]=0;
	}

	for(x=0;x<=100000;x++){
		hourly_rain_data[x]=0;
	}
}


/********* routes the water !! *****************/

/*********** route up **************************/
void route_up()
{
  int x,y,inc, inctot;
  float q, jsat, slope, depth,lowest , tot_ht, slopetot, tot_q, temp_a, q_out, ht, totty;
  float manning=0.03;
  float factor=1;
  float threshold=MIN_Q;
  totty=0;
  inc=1;
  inctot=0;
  
  for(y=reach;y>=1;y--){
    inc=1;    
    while(down_scan[y][inc]>0){
      x=down_scan[y][inc];
      inc++;
            
      ht=elev[x][y];
      
      /** inputting water from the edges **/
      
      if(water_depth[x][y+1]==-9999&&water_depth[x][y]!=-99&&y<(reach-10)){
	/** get slope **/
	discharge[x][y]=(area[x][y]*j_mean)*pow(DX,2);
	water_depth[x][y]=-99;
      }
      else{
	discharge[x][y]+=((j_mean*pow(DX,2))/4);
      }
      
      if(discharge[x][y]>MIN_Q){
	
	slope=0;
	slopetot=0;
	
	if(elev[x][y]>elev[x][y-1]){
	  slope+=pow((elev[x][y]-elev[x][y-1])/root,1);
	  slopetot++;
	}
	if(elev[x][y]>elev[x+1][y-1]){
	  slope+=pow((elev[x][y]-elev[x+1][y-1])/DX,1);
	  slopetot++;
	}
	if(elev[x][y]>elev[x+1][y]){
	  slope+=pow((elev[x][y]-elev[x+1][y])/root,1);
	  slopetot++;
	}
	if(elev[x][y]>elev[x+1][y+1]){
	  slope+=pow((elev[x][y]-elev[x+1][y+1])/root,1);
	  slopetot++;
	}
	if(elev[x][y]>elev[x][y+1]){
	  slope+=pow((elev[x][y]-elev[x][y+1])/DX,1);
	  slopetot++;
	}
	if(elev[x][y]>elev[x-1][y+1]){
	  slope+=pow((elev[x][y]-elev[x-1][y+1])/root,1);
	  slopetot++;
	}
	if(elev[x][y]>elev[x-1][y]){
	  slope+=pow((elev[x][y]-elev[x-1][y])/root,1);
	  slopetot++;
	}
	if(elev[x][y]>elev[x-1][y-1]){
	  slope+=pow((elev[x][y]-elev[x-1][y-1])/DX,1);
	  slopetot++;
	}
	if(slope>0)slope=(slope/slopetot);
	if(slope==0)slope=0.001;
	
	threshold=K*slope*DX;
	if(threshold<MIN_Q)threshold=MIN_Q;
	
      }
      
      
      /* then do surface routing */
      
      
      if(discharge[x][y]>threshold&&water_depth[x-1][y-1]!=-9999&&water_depth[x][y-1]!=-9999&&water_depth[x+1][y-1]!=-9999){
	
	

	/** subtract threshold from discharge - effectively subtracting subsurface flow **/
	discharge[x][y]-=threshold;
	tot_q=0;
	temp_a=0;
	
	/********* first sort out wet height *****/
	/* starting with d/s slopes ****/
	
	if(y==1)slope=0.01;
	depth=discharge[x][y]/(pow(DX,1));
	
	/*** now calc depth. *****/
	if(slope>0.005){
	  
	  depth=(pow((((discharge[x][y]/pow(DX,1))*manning)/
		      pow(slope,0.5)),0.6));
	}
	
	/***** depth added if greater than that there. **/
	
	if(depth>water_depth[x][y])water_depth[x][y]=depth;
	
	/***** now to route discharge d/s ***************/
	tot_ht=elev[x][y]+depth;
	
	if(elev[x-1][y-1]<tot_ht||elev[x][y-1]<tot_ht||elev[x+1][y-1]<tot_ht){
	  
	  if(elev[x-1][y-1]<tot_ht)
	    tot_q+=((tot_ht-elev[x-1][y-1])/root);
	  if(elev[x][y-1]<tot_ht)
	    tot_q+=((tot_ht-elev[x][y-1])/DX);
	  if(elev[x+1][y-1]<tot_ht)
	    tot_q+=((tot_ht-elev[x+1][y-1])/root);
	  
	  
				/**** check to see if base is lower *******/
				/* if not then leave some discharge       */
	  
	  q_out=discharge[x][y];
	  lowest=1000;
	  
	  if(elev[x-1][y-1]>elev[x][y]&&elev[x][y-1]>elev[x][y]&&elev[x+1][y-1]>elev[x][y]){
	    
	    if(elev[x-1][y-1]<lowest)lowest=elev[x-1][y-1];
	    if(elev[x][y-1]<lowest)lowest=elev[x][y-1];
	    if(elev[x+1][y-1]<lowest)lowest=elev[x+1][y-1];
	    q_out=q_out*((tot_ht-lowest)/depth);
	  }
	  
	  /** then route surface water **/
  
	  if(elev[x-1][y-1]<tot_ht){
	    discharge[x-1][y-1]+=q_out*(((tot_ht-elev[x-1][y-1])/root)/tot_q);
	    temp_a+=q_out*(((tot_ht-elev[x-1][y-1])/root)/tot_q);
	    max_flow[x][y][8]+=q_out*(((tot_ht-elev[x-1][y-1])/root)/tot_q);
	  }
	  
	  if(elev[x][y-1]<tot_ht){
	    discharge[x][y-1]+=q_out*(((tot_ht-elev[x][y-1])/DX)/tot_q);
	    temp_a+=q_out*(((tot_ht-elev[x][y-1])/DX)/tot_q);
	    max_flow[x][y][1]+=q_out*(((tot_ht-elev[x][y-1])/DX)/tot_q);
	  }
	  
	  if(elev[x+1][y-1]<tot_ht){
	    discharge[x+1][y-1]+=q_out*(((tot_ht-elev[x+1][y-1])/root)/tot_q);
	    temp_a+=q_out*(((tot_ht-elev[x+1][y-1])/root)/tot_q);
	    max_flow[x][y][2]+=q_out*(((tot_ht-elev[x+1][y-1])/root)/tot_q);
	  }
	  
	  discharge[x][y]=discharge[x][y]-temp_a;
	}
	

	/* add subsurface flow back in **/
	discharge[x][y]+=threshold;
      }
      
      /* then route subsurface flow...*/

      if(discharge[x][y]>0&&water_depth[x-1][y-1]!=-9999&&water_depth[x][y-1]!=-9999&&water_depth[x+1][y-1]!=-9999){
	if(elev[x-1][y-1]<=ht||elev[x][y-1]<=ht||elev[x+1][y-1]<=ht){
	  
	  
	  tot_q=0;
	  temp_a=0;
	  q_out=discharge[x][y];
	  
	  if(elev[x-1][y-1]<ht)
	    tot_q+=pow(((ht-elev[x-1][y-1])/root),factor);
	  if(elev[x][y-1]<ht)
	    tot_q+=pow(((ht-elev[x][y-1])/DX),factor);
	  if(elev[x+1][y-1]<ht)
	    tot_q+=pow(((ht-elev[x+1][y-1])/root),factor);
	  
	  if(elev[x-1][y-1]<ht){
	    discharge[x-1][y-1]+=q_out*(((ht-elev[x-1][y-1])/root)/tot_q);
	    temp_a+=q_out*(((ht-elev[x-1][y-1])/root)/tot_q);
	  }
	  
	  if(elev[x][y-1]<ht){
	    discharge[x][y-1]+=q_out*(((ht-elev[x][y-1])/DX)/tot_q);
	    temp_a+=q_out*(((ht-elev[x][y-1])/DX)/tot_q);
	  }
	  
	  if(elev[x+1][y-1]<ht){
	    discharge[x+1][y-1]+=q_out*(((ht-elev[x+1][y-1])/root)/tot_q);
	    temp_a+=q_out*(((ht-elev[x+1][y-1])/root)/tot_q);
	  }
	  
	  discharge[x][y]=discharge[x][y]-temp_a;
	  
	}
      }   
    }
  }
}




/*********** route down  ***********************/

void route_down(cycle)
float cycle;
{
  int x,y,inc, inctot;
  float q, jsat, slope, depth,lowest , tot_ht, slopetot, tot_q, temp_a, q_out, ht,totty;
  float manning=0.03;
  float factor=1;
  float threshold=MIN_Q;
  totty=0;
  inc=1;
  inctot=0;
  
  for(y=1;y<=reach;y++){
    
    inctot+=inc;
    inc=1;
    
    while(down_scan[y][inc]>0){
      x=down_scan[y][inc];
      inc++;
      
      
      ht=elev[x][y];
      
      
      /* route water if not enough to flow */
      
      if(water_depth[x][y-1]==-9999&&water_depth[x][y]!=-99){
	
	discharge[x][y]=(area[x][y]*j_mean)*pow(DX,2);
	water_depth[x][y]=-99;
	
      }
      else{
	discharge[x][y]+=((j_mean*pow(DX,2))/4);
      }
      
      if(discharge[x][y]>MIN_Q){
	
	slope=0;
	slopetot=0;
	
	if(elev[x][y]>elev[x][y-1]){
	  slope+=pow((elev[x][y]-elev[x][y-1])/root,1);
	  slopetot++;
	}
	if(elev[x][y]>elev[x+1][y-1]){
	  slope+=pow((elev[x][y]-elev[x+1][y-1])/DX,1);
	  slopetot++;
	}
	if(elev[x][y]>elev[x+1][y]){
	  slope+=pow((elev[x][y]-elev[x+1][y])/root,1);
	  slopetot++;
	}
	if(elev[x][y]>elev[x+1][y+1]){
	  slope+=pow((elev[x][y]-elev[x+1][y+1])/root,1);
	  slopetot++;
	}
	if(elev[x][y]>elev[x][y+1]){
	  slope+=pow((elev[x][y]-elev[x][y+1])/DX,1);
	  slopetot++;
	}
	if(elev[x][y]>elev[x-1][y+1]){
	  slope+=pow((elev[x][y]-elev[x-1][y+1])/root,1);
	  slopetot++;
	}
	if(elev[x][y]>elev[x-1][y]){
	  slope+=pow((elev[x][y]-elev[x-1][y])/root,1);
	  slopetot++;
	}
	if(elev[x][y]>elev[x-1][y-1]){
	  slope+=pow((elev[x][y]-elev[x-1][y-1])/DX,1);
				slopetot++;
	}
	if(slope>0)slope=(slope/slopetot);
	if(slope==0)slope=0.001;
	
	threshold=K*slope*DX;
	if(threshold<MIN_Q)threshold=MIN_Q;
	
      }
      
      /* then do surface routing */
      
      
      if(discharge[x][y]>threshold&&water_depth[x-1][y+1]!=-9999&&water_depth[x][y+1]!=-9999&&water_depth[x+1][y+1]!=-9999){
	
	
	discharge[x][y]-=threshold;
	tot_q=0;
	
	temp_a=0;
	
	
	/********* first sort out wet height *****/
	/* starting with d/s slopes ****/
	
	/* using average slope... */
	
	
	if(y==reach)slope=0.01;
	
	depth=discharge[x][y]/pow(DX,1);
	
	/*** now calc depth. *****/
	if(slope>0.005){
	  
	  depth=(pow((((discharge[x][y]/pow(DX,1))*manning)/
		      pow(slope,0.5)),0.6));
	}
	
	/***** depth added if greater than that there. **/
	
	if(depth>water_depth[x][y])water_depth[x][y]=depth;
	
	/***** now to route discharge d/s ***************/
	
	tot_ht=elev[x][y]+depth;
	
	if(elev[x-1][y+1]<tot_ht||elev[x][y+1]<tot_ht||elev[x+1][y+1]<tot_ht){
	  
	  if(elev[x-1][y+1]<tot_ht)
	    tot_q+=((tot_ht-elev[x-1][y+1])/root);
	  if(elev[x][y+1]<tot_ht)
	    tot_q+=((tot_ht-elev[x][y+1])/DX);
	  if(elev[x+1][y+1]<tot_ht)
	    tot_q+=((tot_ht-elev[x+1][y+1])/root);
	  
	  
				/**** check to see if base is lower *******/
				/* if not then leave some discharge       */
	  
	  q_out=discharge[x][y];
	  lowest=1000;
	  
	  if(elev[x-1][y+1]>elev[x][y]&&elev[x][y+1]>elev[x][y]&&elev[x+1][y+1]>elev[x][y]){
	    
	    if(elev[x-1][y+1]<lowest)lowest=elev[x-1][y+1];
	    if(elev[x][y+1]<lowest)lowest=elev[x][y+1];
	    if(elev[x+1][y+1]<lowest)lowest=elev[x+1][y+1];
	    q_out=q_out*((tot_ht-lowest)/depth);
	  }
	  
	  
	  if(elev[x-1][y+1]<tot_ht){
	    discharge[x-1][y+1]+=q_out*(((tot_ht-elev[x-1][y+1])/root)/tot_q);
	    temp_a+=q_out*(((tot_ht-elev[x-1][y+1])/root)/tot_q);
	    max_flow[x][y][6]+=q_out*(((tot_ht-elev[x-1][y+1])/root)/tot_q);
	  }
	  
	  if(elev[x][y+1]<tot_ht){
	    discharge[x][y+1]+=q_out*(((tot_ht-elev[x][y+1])/DX)/tot_q);
	    temp_a+=q_out*(((tot_ht-elev[x][y+1])/DX)/tot_q);
	    max_flow[x][y][5]+=q_out*(((tot_ht-elev[x][y+1])/DX)/tot_q);
	  }
	  
	  if(elev[x+1][y+1]<tot_ht){
	    discharge[x+1][y+1]+=q_out*(((tot_ht-elev[x+1][y+1])/root)/tot_q);
	    temp_a+=q_out*(((tot_ht-elev[x+1][y+1])/root)/tot_q);
	    max_flow[x][y][4]+=q_out*(((tot_ht-elev[x+1][y+1])/root)/tot_q);
	  }
	  if(y==reach-10)totty+=temp_a;
	  discharge[x][y]=discharge[x][y]-temp_a;
	}
	discharge[x][y]+=threshold;
	
      }
      
      if(discharge[x][y]>0&&water_depth[x-1][y+1]!=-9999&&water_depth[x][y+1]!=-9999&&water_depth[x+1][y+1]!=-9999){
	if(elev[x-1][y+1]<=ht||elev[x][y+1]<=ht||elev[x+1][y+1]<=ht){
	  
	  
	  tot_q=0;
	  temp_a=0;
	  q_out=discharge[x][y];
	  
	  if(elev[x-1][y+1]<ht)
	    tot_q+=pow(((ht-elev[x-1][y+1])/root),factor);
	  if(elev[x][y+1]<ht)
	    tot_q+=pow(((ht-elev[x][y+1])/DX),factor);
	  if(elev[x+1][y+1]<ht)
	    tot_q+=pow(((ht-elev[x+1][y+1])/root),factor);
	  
	  if(elev[x-1][y+1]<ht){
	    discharge[x-1][y+1]+=q_out*(((ht-elev[x-1][y+1])/root)/tot_q);
	    temp_a+=q_out*(((ht-elev[x-1][y+1])/root)/tot_q);
	  }
	  
	  if(elev[x][y+1]<ht){
	    discharge[x][y+1]+=q_out*(((ht-elev[x][y+1])/DX)/tot_q);
	    temp_a+=q_out*(((ht-elev[x][y+1])/DX)/tot_q);
	  }
	  
	  if(elev[x+1][y+1]<ht){
	    discharge[x+1][y+1]+=q_out*(((ht-elev[x+1][y+1])/root)/tot_q);
	    temp_a+=q_out*(((ht-elev[x+1][y+1])/root)/tot_q);
	  }
				/*	if(y==reach-150)totty+=temp_a;*/
	  discharge[x][y]=discharge[x][y]-temp_a;
	  
	}
      }
      
      
    }
  }
  
  /*printf("%f %f %f\n",cycle, totty, j_mean*3000000); */
}


/********************* route from left to right *********************/


float route_lr()
{
	int x,y,inc, inctot;
	float jsat, q, ht, slope, depth,lowest , tot_ht, tot_q, temp_a, q_out, slopetot;
	float manning=0.03;
	float factor=1;
	float totty=0;
	float threshold=MIN_Q;
	y=y_min;
	inctot=0;
	inc=1;
        
        for(x=1;x<=(xsection);x++){

	        inctot+=inc;
		inc=1;
	
		while(cross_scan[x][inc]>0){

		y=cross_scan[x][inc];
		inc++;


/*****************/

/*****************/

		ht=elev[x][y];
		
		if(water_depth[x-1][y]==-9999&&water_depth[x][y]!=-99){

			discharge[x][y]=(area[x][y]*j_mean)*pow(DX,2);
			water_depth[x][y]=-99;
		}
     		else{
		  discharge[x][y]+=((j_mean*pow(DX,2))/4);
		}
		if(discharge[x][y]>MIN_Q){

			slope=0;
			slopetot=0;

			if(elev[x][y]>elev[x][y-1]){
				slope+=pow((elev[x][y]-elev[x][y-1])/root,1);
				slopetot++;
			}
			if(elev[x][y]>elev[x+1][y-1]){
				slope+=pow((elev[x][y]-elev[x+1][y-1])/DX,1);
				slopetot++;
			}
			if(elev[x][y]>elev[x+1][y]){
				slope+=pow((elev[x][y]-elev[x+1][y])/root,1);
				slopetot++;
			}
			if(elev[x][y]>elev[x+1][y+1]){
				slope+=pow((elev[x][y]-elev[x+1][y+1])/root,1);
				slopetot++;
			}
			if(elev[x][y]>elev[x][y+1]){
				slope+=pow((elev[x][y]-elev[x][y+1])/DX,1);
				slopetot++;
			}
			if(elev[x][y]>elev[x-1][y+1]){
				slope+=pow((elev[x][y]-elev[x-1][y+1])/root,1);
				slopetot++;
			}
			if(elev[x][y]>elev[x-1][y]){
				slope+=pow((elev[x][y]-elev[x-1][y])/root,1);
				slopetot++;
			}
			if(elev[x][y]>elev[x-1][y-1]){
				slope+=pow((elev[x][y]-elev[x-1][y-1])/DX,1);
				slopetot++;
			}
			if(slope>0)slope=(slope/slopetot);
			if(slope==0)slope=0.001;

			threshold=K*slope*DX;
			if(threshold<MIN_Q)threshold=MIN_Q;

		}


		/* route water if not enough to flow */

		/* then do surface routing */

		if(discharge[x][y]>threshold&&water_depth[x+1][y-1]!=-9999&&water_depth[x+1][y]!=-9999&&water_depth[x+1][y+1]!=-9999){
			discharge[x][y]-=threshold;

			tot_q=0;

			temp_a=0;

			/********* first sort out wet height *****/
			/* starting with d/s slopes ****/

/* using average slope... */


/*
			lowest=1000;
			if(elev[x+1][y-1]<lowest)lowest=elev[x+1][y-1];
			if(elev[x+1][y]<lowest)lowest=elev[x+1][y];
			if(elev[x+1][y+1]<lowest)lowest=elev[x+1][y+1];

			slope=elev[x][y]-lowest;
*/

			/*** now calc depth. *****/

			if(x==xsection)slope=0.01;

			depth=(discharge[x][y]/pow(DX,1));
			if(slope>0.005){

				depth=(pow((((discharge[x][y]/pow(DX,1))*manning)/
				pow(slope,0.5)),0.6));
			}

			/***** depth added if greater than that there. **/

			if(depth>water_depth[x][y])water_depth[x][y]=depth;

			/***** now to route discharge d/s ***************/

			tot_ht=elev[x][y]+depth;

			if(elev[x+1][y-1]<tot_ht||elev[x+1][y]<tot_ht||elev[x+1][y+1]<tot_ht){

				if(elev[x+1][y-1]<tot_ht)
					tot_q+=((tot_ht-elev[x+1][y-1])/root);
				if(elev[x+1][y]<tot_ht)
					tot_q+=((tot_ht-elev[x+1][y])/DX);
				if(elev[x+1][y+1]<tot_ht)
					tot_q+=((tot_ht-elev[x+1][y+1])/root);


				/**** check to see if base is lower *******/
				/* if not then leave some discharge       */

				q_out=discharge[x][y];
				lowest=1000;

				if(elev[x+1][y-1]>elev[x][y]&&elev[x+1][y]>elev[x][y]&&elev[x+1][y+1]>elev[x][y]){
					/****/
					if(elev[x+1][y-1]<lowest)lowest=elev[x+1][y-1];
					if(elev[x+1][y]<lowest)lowest=elev[x+1][y];
					if(elev[x+1][y+1]<lowest)lowest=elev[x+1][y+1];
					q_out=q_out*((tot_ht-lowest)/depth);
				}


				if(elev[x+1][y-1]<tot_ht){
					discharge[x+1][y-1]+=q_out*(((tot_ht-elev[x+1][y-1])/root)/tot_q);
					temp_a+=q_out*(((tot_ht-elev[x+1][y-1])/root)/tot_q);
					max_flow[x][y][2]+=q_out*(((tot_ht-elev[x+1][y-1])/root)/tot_q);
				}

				if(elev[x+1][y]<tot_ht){
					discharge[x+1][y]+=q_out*(((tot_ht-elev[x+1][y])/DX)/tot_q);
					temp_a+=q_out*(((tot_ht-elev[x+1][y])/DX)/tot_q);
					max_flow[x][y][3]+=q_out*(((tot_ht-elev[x+1][y])/DX)/tot_q);
				}

				if(elev[x+1][y+1]<tot_ht){
					discharge[x+1][y+1]+=q_out*(((tot_ht-elev[x+1][y+1])/root)/tot_q);
					temp_a+=q_out*(((tot_ht-elev[x+1][y+1])/root)/tot_q);
					max_flow[x][y][4]+=q_out*(((tot_ht-elev[x+1][y+1])/root)/tot_q);
				}
				if(x==xsection)totty+=temp_a;
				discharge[x][y]=discharge[x][y]-temp_a;
			}

			discharge[x][y]+=threshold;

		}

		if(discharge[x][y]>0&&water_depth[x+1][y-1]!=-9999&&water_depth[x+1][y]!=-9999&&water_depth[x+1][y+1]!=-9999){
			if(elev[x+1][y-1]<=ht||elev[x+1][y]<=ht||elev[x+1][y+1]<=ht){

				tot_q=0;
				temp_a=0;
				q_out=discharge[x][y];

				if(elev[x+1][y-1]<ht)
					tot_q+=((ht-elev[x+1][y-1])/root);
				if(elev[x+1][y]<ht)
					tot_q+=(ht-elev[x+1][y])/DX;
				if(elev[x+1][y+1]<ht)
					tot_q+=((ht-elev[x+1][y+1])/root);

				if(elev[x+1][y-1]<ht){
					discharge[x+1][y-1]+=q_out*(((ht-elev[x+1][y-1])/root)/tot_q);
					temp_a+=q_out*(((ht-elev[x+1][y-1])/root)/tot_q);
				}

				if(elev[x+1][y]<ht){
					discharge[x+1][y]+=q_out*(((ht-elev[x+1][y])/DX)/tot_q);
					temp_a+=q_out*(((ht-elev[x+1][y])/DX)/tot_q);
				}

				if(elev[x+1][y+1]<ht){
					discharge[x+1][y+1]+=q_out*(((ht-elev[x+1][y+1])/root)/tot_q);
					temp_a+=q_out*(((ht-elev[x+1][y+1])/root)/tot_q);
				}

				discharge[x][y]=discharge[x][y]-temp_a;

			}
		}

		}
	}
	printf("%d %f %f %f ",year,((year_inc*5256000)-5256000)+cycle, totty, j_mean*3000000);
	return totty;
}


/********************* route from right to left *********************/




void route_rl()
{
       
	int x,y,inc;
	float jsat, q, ht, slope, depth,lowest , tot_ht, tot_q, temp_a, q_out, slopetot;
	float manning=0.03;
	float factor=1;
	float threshold=MIN_Q;
	y=y_min+1;

        for(x=xsection;x>=1;x--){

		inc=1;

		while(cross_scan[x][inc]>0){
		y=cross_scan[x][inc];
		inc++;

		ht=elev[x][y];

		if(water_depth[x+1][y]==-9999&&water_depth[x][y]!=-99){
			discharge[x][y]=(area[x][y]*j_mean)*pow(DX,2);
			water_depth[x][y]=-99;
		}
		else{
		  discharge[x][y]+=((j_mean*pow(DX,2))/4);
		}

		if(discharge[x][y]>MIN_Q){

			slope=0;
			slopetot=0;

			if(elev[x][y]>elev[x][y-1]){
				slope+=pow((elev[x][y]-elev[x][y-1])/root,1);
				slopetot++;
			}
			if(elev[x][y]>elev[x+1][y-1]){
				slope+=pow((elev[x][y]-elev[x+1][y-1])/DX,1);
				slopetot++;
			}
			if(elev[x][y]>elev[x+1][y]){
				slope+=pow((elev[x][y]-elev[x+1][y])/root,1);
				slopetot++;
			}
			if(elev[x][y]>elev[x+1][y+1]){
				slope+=pow((elev[x][y]-elev[x+1][y+1])/root,1);
				slopetot++;
			}
			if(elev[x][y]>elev[x][y+1]){
				slope+=pow((elev[x][y]-elev[x][y+1])/DX,1);
				slopetot++;
			}
			if(elev[x][y]>elev[x-1][y+1]){
				slope+=pow((elev[x][y]-elev[x-1][y+1])/root,1);
				slopetot++;
			}
			if(elev[x][y]>elev[x-1][y]){
				slope+=pow((elev[x][y]-elev[x-1][y])/root,1);
				slopetot++;
			}
			if(elev[x][y]>elev[x-1][y-1]){
				slope+=pow((elev[x][y]-elev[x-1][y-1])/DX,1);
				slopetot++;
			}
			if(slope>0)slope=(slope/slopetot);
			if(slope==0)slope=0.001;

			threshold=K*slope*DX;
			if(threshold<MIN_Q)threshold=MIN_Q;

		}

		/* route water if not enough to flow */

		/* then do surface routing */

		if(discharge[x][y]>threshold&&water_depth[x-1][y-1]!=-9999&&water_depth[x-1][y]!=-9999&&water_depth[x-1][y+1]!=-9999){

			discharge[x][y]-=threshold;


			tot_q=0;

			temp_a=0;

			/********* first sort out wet height *****/
			/* starting with d/s slopes ****/

			/*** now calc depth. *****/

			if(x==1)slope=0.01;

			depth=(discharge[x][y]/pow(DX,1));

			if(slope>0.005){

				depth=(pow((((discharge[x][y]/pow(DX,1))*manning)/
				pow(slope,0.5)),0.6));
			}
			/***** depth added if greater than that there. **/

			if(depth>water_depth[x][y])water_depth[x][y]=depth;

			/***** now to route discharge d/s ***************/

			tot_ht=elev[x][y]+depth;

			if(elev[x-1][y-1]<tot_ht||elev[x-1][y]<tot_ht||elev[x-1][y+1]<tot_ht){

				if(elev[x-1][y-1]<tot_ht)
					tot_q+=((tot_ht-elev[x-1][y-1])/root);
				if(elev[x-1][y]<tot_ht)
					tot_q+=((tot_ht-elev[x-1][y])/DX);
				if(elev[x-1][y+1]<tot_ht)
					tot_q+=((tot_ht-elev[x-1][y+1])/root);


				/**** check to see if base is lower *******/
				/* if not then leave some discharge       */

				q_out=discharge[x][y];
				lowest=1000;

				if(elev[x-1][y-1]>elev[x][y]&&elev[x-1][y]>elev[x][y]&&elev[x-1][y+1]>elev[x][y]){
					/****/
					if(elev[x-1][y-1]<lowest)lowest=elev[x-1][y-1];
					if(elev[x-1][y]<lowest)lowest=elev[x-1][y];
					if(elev[x-1][y+1]<lowest)lowest=elev[x-1][y+1];
					q_out=q_out*((tot_ht-lowest)/depth);
				}


				if(elev[x-1][y-1]<tot_ht){
					discharge[x-1][y-1]+=q_out*(((tot_ht-elev[x-1][y-1])/root)/tot_q);
					temp_a+=q_out*(((tot_ht-elev[x-1][y-1])/root)/tot_q);
					max_flow[x][y][8]+=q_out*(((tot_ht-elev[x-1][y-1])/root)/tot_q);
				}

				if(elev[x-1][y]<tot_ht){
					discharge[x-1][y]+=q_out*(((tot_ht-elev[x-1][y])/DX)/tot_q);
					temp_a+=q_out*(((tot_ht-elev[x-1][y])/DX)/tot_q);
					max_flow[x][y][7]+=q_out*(((tot_ht-elev[x-1][y])/DX)/tot_q);
				}

				if(elev[x-1][y+1]<tot_ht){
					discharge[x-1][y+1]+=q_out*(((tot_ht-elev[x-1][y+1])/root)/tot_q);
					temp_a+=q_out*(((tot_ht-elev[x-1][y+1])/root)/tot_q);
					max_flow[x][y][6]+=q_out*(((tot_ht-elev[x-1][y+1])/root)/tot_q);
				}

				discharge[x][y]=discharge[x][y]-temp_a;
			}

			discharge[x][y]+=threshold;

		}
		if(discharge[x][y]>0&&water_depth[x-1][y-1]!=-9999&&water_depth[x-1][y]!=-9999&&water_depth[x-1][y+1]!=-9999){
			if(elev[x-1][y-1]<=ht||elev[x-1][y]<=ht||elev[x-1][y+1]<=ht){


				tot_q=0;
				temp_a=0;
				q_out=discharge[x][y];

				if(elev[x-1][y-1]<ht)
					tot_q+=((ht-elev[x-1][y-1])/root);
				if(elev[x-1][y]<ht)
					tot_q+=((ht-elev[x-1][y])/DX);
				if(elev[x-1][y+1]<ht)
					tot_q+=((ht-elev[x-1][y+1])/root);

				if(elev[x-1][y-1]<ht){
					discharge[x-1][y-1]+=q_out*(((ht-elev[x-1][y-1])/root)/tot_q);
					temp_a+=q_out*(((ht-elev[x-1][y-1])/root)/tot_q);
				}

				if(elev[x-1][y]<ht){
					discharge[x-1][y]+=q_out*(((ht-elev[x-1][y])/DX)/tot_q);
					temp_a+=q_out*(((ht-elev[x-1][y])/DX)/tot_q);
				}

				if(elev[x-1][y+1]<ht){
					discharge[x-1][y+1]+=q_out*(((ht-elev[x-1][y+1])/root)/tot_q);
					temp_a+=q_out*(((ht-elev[x-1][y+1])/root)/tot_q);
				}

				discharge[x][y]=discharge[x][y]-temp_a;


			}
		}


		}
	}
}


/********************* calcualte flow direcitons ****/

void calc_flow_direction()
{
  int x,y,n,inc, inctot,val;
  float tot;
  
  for(y=reach;y>=1;y--){
    inc=1;    
    while(down_scan[y][inc]>0){
      x=down_scan[y][inc];
      inc++;

      /** calculate flow out direction **/
      tot=0;
      for(n=1;n<=8;n++){
	if(max_flow[x][y][n]>tot){
	  val=n;
	  tot=max_flow[x][y][n];
	  max_flow[x][y][0]=tot; /* stores max discharge out in 0 of max_flow array.*/
	}
      }
      if(tot>0)flow_dir[x][y][1]=val;

      /** calculate flow in direction  **/

      tot=0;

      if(max_flow[x][y-1][5]>tot){
	val=1;
	tot=max_flow[x][y-1][5];
      }

      if(max_flow[x+1][y-1][6]>tot){
	val=2;
	tot=max_flow[x+1][y-1][6];
      }

      if(max_flow[x+1][y][7]>tot){
	val=3;
	tot=max_flow[x+1][y][7];
      }

      if(max_flow[x+1][y+1][8]>tot){
	val=4;
	tot=max_flow[x+1][y+1][8];
      }

      if(max_flow[x][y+1][1]>tot){
	val=5;
	tot=max_flow[x][y+1][1];
      }

      if(max_flow[x-1][y+1][2]>tot){
	val=6;
	tot=max_flow[x-1][y+1][2];
      }

      if(max_flow[x-1][y][3]>tot){
	val=7;
	tot=max_flow[x-1][y][3];
      }

      if(max_flow[x-1][y-1][4]>tot){
	val=8;
	tot=max_flow[x-1][y-1][4];
      }


      if(tot>0)flow_dir[x][y][0]=val;

    }
  }
}





/******************* eroding function *******************************/

void erode()
{
  float tot_ht, ht, tot, depth, amount,t, tempb, tempbmax, total;
  int x,y,n, inc, loopinc,q,xtemp,ytemp,temp_centre,p;
  int dx = 1;
  float max;
  float exp=1.5;
  float total2, total3;
  float totty=0;
  float depth_threshold=0.0001;
  double oldtot, newtot, tempgrid;
  double gtot[11];
  tempb=0;
  
  for(n=0;n<=G_MAX;n++){
    gtot[n]=0;
  }	
  
  do{
    
    for(x=0;x<=grain_array_tot;x++){
      for(y=0;y<=G_MAX;y++){
	for(p=0;p<=(TRACERS-1);p++){
	  temp[x][y][p]=0;
	}
      }
    }
    
    max=0;
    
    for(y=1;y<reach;y++){
      
      inc=1;
      while(down_scan[y][inc]>0){
	x=down_scan[y][inc];
	inc++;	
	
	/** check to see if under water **/
	
	if(water_depth[x][y]>depth_threshold&&x>1&&x<xsection){
	  
	  tot_ht=elev[x][y]+water_depth[x][y];
	  depth=water_depth[x][y];
	  ht=elev[x][y];
	  tempb=0;
	  
	  for(n=1;n<=8;n++){
	    for(loopinc=0;loopinc<=G_MAX;loopinc++){
	      dist[n][loopinc]=0;
	    }
	  }



	  /*************** then incision/depo ************/
	  
	  
	  if(elev[x-1][y-1]<ht&&(water_depth[x-1][y-1]+elev[x-1][y-1])<=tot_ht&&elev[x-1][y-1]>0){
	    erode2(1,t,depth,((ht-elev[x-1][y-1])/root));
	  }
	  if(elev[x][y-1]<ht&&(water_depth[x][y-1]+elev[x][y-1])<=tot_ht&&elev[x][y-1]>0){
	    erode2(2,t,depth,((ht-elev[x][y-1])/DX));
	  }
	  if(elev[x+1][y-1]<ht&&(water_depth[x+1][y-1]+elev[x+1][y-1])<=tot_ht&&elev[x+1][y-1]>0){
	    erode2(3,t,depth,((ht-elev[x+1][y-1])/root));
	  }
	  if(elev[x+1][y]<ht&&(water_depth[x+1][y]+elev[x+1][y])<=tot_ht&&elev[x+1][y]>0){
	    erode2(4,t,depth,((ht-elev[x+1][y])/DX));
	  }
	  if(elev[x+1][y+1]<ht&&(water_depth[x+1][y+1]+elev[x+1][y+1])<=tot_ht&&elev[x+1][y+1]>0){
	    erode2(5,t,depth,((ht-elev[x+1][y+1])/root));
	  }
	  if(elev[x][y+1]<ht&&(water_depth[x][y+1]+elev[x][y+1])<=tot_ht&&elev[x][y+1]>0){
	    erode2(6,t,depth,((ht-elev[x][y+1])/DX));
	  }
	  if(elev[x-1][y+1]<ht&&(water_depth[x-1][y+1]+elev[x-1][y+1])<=tot_ht&&elev[x-1][y+1]>0){
	    erode2(7,t,depth,((ht-elev[x-1][y+1])/root));
	  }
	  if(elev[x-1][y]<ht&&(water_depth[x-1][y]+elev[x-1][y])<=tot_ht&&elev[x-1][y]>0){
	    erode2(8,t,depth,((ht-elev[x-1][y])/DX));
	  }
	  
	  
	  /************************************************************************************************/
	  
	  /* calls erode 3, with amount, dist category, grid category (for grain sizes) and x+y's */
	  erode3(0,3,x,y);
	  erode3(1,4,x,y);
	  erode3(2,5,x,y);
	  erode3(3,6,x,y);
	  erode3(4,7,x,y);
	  erode3(5,8,x,y);
	  erode3(6,9,x,y);
	  erode3(7,10,x,y);
	  erode3(8,11,x,y);
	  erode3(9,12,x,y);
	  erode3(10,13,x,y);
	 
	  
	  /*** try new tempb approach....  *******************************************/
	  
	  tempb=0;
	  if(index[x][y]!=-9999){
	    for(loopinc=1;loopinc<=G_MAX;loopinc++){
	      for(p=0;p<=(TRACERS-1);p++){
		tempb+=temp[CENTRE][loopinc][p];
	      }
	    }
	  }
	  
	  
	  if(fabs(tempb)>ERODEFACTOR){
	    
	    y=reach;
	    x=xsection;
	    
	  }
	  
	}
      }
    }
    
    
    if(fabs(tempb)>ERODEFACTOR)time_factor=time_factor/2;
    if(fabs(tempb)<ERODEFACTOR)time_factor=time_factor*2;
    
  }while(fabs(tempb)>ERODEFACTOR);
  

  /* from here on is really only an update the values routine...*/

  
  for(y=1;y<=reach;y++){
    
    inc=1;
    while(down_scan[y][inc]>0){
      x=down_scan[y][inc];
      inc++;	


      if(water_depth[x][y]>0){
	xtemp=x;
	ytemp=y;
	for(q=1;q<=9;q++){
	  if(q==1){
	    x=xtemp;
	    y=ytemp;
	  }
	  if(q==2){
	    x=xtemp-1;
	    y=ytemp-1;
	  }
	  if(q==3){
	    x=xtemp;
	    y=ytemp-1;
	  }
	  if(q==4){
	    x=xtemp+1;
	    y=ytemp-1;
	  }
	  if(q==5){
	    x=xtemp+1;
	    y=ytemp;
	  }
	  if(q==6){
	    x=xtemp+1;
	    y=ytemp+1;
	  }
	  if(q==7){
	    x=xtemp;
	    y=ytemp+1;
	  }
	  if(q==8){
	    x=xtemp-1;
	    y=ytemp+1;
	  }
	  if(q==9){
	    x=xtemp-1;
	    y=ytemp;
	  }			  
	  temp_centre=CENTRE;
	  if(index[x][y]!=-9999&&y<=reach&&y>=1&&x>=1&&x<xsection){
	    
	    
				/********* if eroding then adding to active layer ******/
	    
				/**** this next line(s) to ensure that grass (and others) is directly removed from the system. **/
	    for(p=0;p<=(TRACERS-1);p++){
	      if(temp[temp_centre][0][p]>0)temp[temp_centre][0][p]=0;
	      if(temp[temp_centre][G_MAX][p]!=0){
		temp[temp_centre][G_MAX][p]=0;	
		printf("ARSE ");
	      }
	    }
	    
	    total3=0;
	    total2=0;
	    oldtot=0;
	    newtot=0;
	    
	    for(loopinc=0;loopinc<=G_MAX;loopinc++){
	      for(p=0;p<=(TRACERS-1);p++){
		if((grain[temp_centre][loopinc][p]+temp[temp_centre][loopinc][p])<0){ /* trap to remove -ve's */
		  printf(" to %f %d %d %d\n",(grain[CENTRE][loopinc][p]+temp[CENTRE][loopinc][p])*100000,loopinc,x,y);
		  if(grain[temp_centre][loopinc][p]<0)printf(" fro %f\n",grain[temp_centre][loopinc][p]*100000);
		  if(grain[CENTRE][loopinc][p]>0){
		    temp[CENTRE][loopinc][p]=0-grain[CENTRE][loopinc][p];
		  }
		  else{
		    temp[CENTRE][loopinc][p]=0;
		    grain[CENTRE][loopinc][p]=0;
		  }
		}
		/*Bit to add material back in...*/
		/*
		if(x==5&&y==100){
		  if(temp_grain[loopinc]<0)temp_grain[loopinc]=0;
		  temp[CENTRE][loopinc][p]+=temp_grain[loopinc];
		  temp_grain[loopinc]=0;
		}
		*/
		/*************/
		if(loopinc>0)oldtot+=grain[temp_centre][loopinc][p];
		grain[temp_centre][loopinc][p]+=temp[temp_centre][loopinc][p];
		if(loopinc>0)newtot+=grain[temp_centre][loopinc][p];
		/*if(loopinc>0&&p==0)gtot[loopinc]+=temp[temp_centre][loopinc][p];*/
		temp[temp_centre][loopinc][p]=0;
	      }
	    }
	    
				/*** here to update by double precision... */
	    
	    
	    elev[x][y]+=(newtot-oldtot);
	    
	    sort_active(x,y);
	    
	  }

	  
	  /*if(water_depth[x][y]>0)water_depth[x][y]=0;to stop it checking twice*/
	  
	}
	x=xtemp;
	y=ytemp;

	if(x==xsection){
	  for(n=1;n<G_MAX;n++){
	    for(p=0;p<=TRACERS-1;p++){
	      gtot[n]+=temp[CENTRE][n][p];
	    }
	  }
	}
	
      }
      
      /**** zero's water height and sets discharge in. ***/
      /** now being done at lateral **/
      
    }
  }
  /*printf("%f\n",gtot*DX*DX);*/
  printf("%f %f %f %f %f %f %f %f %f\n",gtot[1]*DX*DX,gtot[2]*DX*DX,gtot[3]*DX*DX,gtot[4]*DX*DX,gtot[5]*DX*DX,gtot[6]*DX*DX,gtot[7]*DX*DX,gtot[8]*DX*DX,gtot[9]*DX*DX);
  for(n=1;n<=G_MAX-1;n++)temp_grain[n]=gtot[n];
}


/******************* lateral erosion routine *************************************/



void lateral()
{

  int x,y,n,inc, inctot,val;
  int hl1x,hl1y,
    hl2x,hl2y,
    hl3x,hl3y,
    hl4x,hl4y,

    ml1x,ml1y, ml1x_rec, ml1y_rec,
    ml2x,ml2y,
    ml3x,ml3y,

    el1x,el1y, el1x_rec, el1y_rec, 
    el2x,el2y, el2x_rec, el2y_rec,
    el3x,el3y,

    hr1x,hr1y,
    hr2x,hr2y,
    hr3x,hr3y,
    hr4x,hr4y,

    mr1x,mr1y, mr1x_rec, mr1y_rec,
    mr2x,mr2y,
    mr3x,mr3y,

    er1x,er1y, er1x_rec, er1y_rec,
    er2x,er2y, er2x_rec, er2y_rec,
    er3x,er3y,

    hl,ml,el,hr,mr,er;

  float tot,amount;

  /* calculate lateral erosion */
  /*

  */

  for(y=reach;y>=1;y--){
    inc=1;    
    while(down_scan[y][inc]>0){
      x=down_scan[y][inc];
      inc++;


      if(water_depth[x][y]>0.0001&&flow_dir[x][y][0]!=0&&flow_dir[x][y][1]!=0&&(flow_dir[x][y][0]-flow_dir[x][y][1])!=4&&(flow_dir[x][y][0]-flow_dir[x][y][1])!=-4){

	/*printf("\n %d %d",x,y);*/



	hl=0;ml=0;el=0;hr=0;mr=0;er=0;

	/* now do lateral erosion bit 
	 this determines the flow direction in - */



	if(flow_dir[x][y][0]==1){
	  
	  ml=3;
	  ml1x=x;	  ml1y=y+1;
	  ml2x=x-1;       ml2y=y+1;
	  ml3x=x-1;	  ml3y=y;

	  ml1x_rec=x+1;      ml1y_rec=y-1;
	  el1x_rec=x+1;      el1y_rec=y-1;
	  el2x_rec=x+1;      el2y_rec=y;

	  el=4;
	  /* no need for easy right or left, as co-ords same as ml!*/
 
	  mr=7;
	  mr1x=x;	  mr1y=y+1;
	  mr2x=x+1;	  mr2y=y+1;
	  mr3x=x+1;	  mr3y=y;	  

	  mr1x_rec=x-1;      mr1y_rec=y-1;
	  er1x_rec=x-1;      er1y_rec=y-1;
	  er2x_rec=x-1;      er2y_rec=y;

	  er=6;
	}



	if(flow_dir[x][y][0]==2){
  
	  ml=4;
	  ml1x=x-1;	  ml1y=y+1;
	  ml2x=x-1;        ml2y=y;
	  ml3x=x-1;	  ml3y=y-1;
	 
	  ml1x_rec=x+1;      ml1y_rec=y;
	  el1x_rec=x+1;      el1y_rec=y;
	  el2x_rec=x+1;      el2y_rec=y+1;
 
	  el=5;
	  /* no need for easy right or left, as co-ords same as ml!*/
	  
	  mr=8;
	  mr1x=x-1;	  mr1y=y+1;
	  mr2x=x;	  mr2y=y+1;
	  mr3x=x+1;	  mr3y=y+1;	  

	  mr1x_rec=x;      mr1y_rec=y-1;
	  er1x_rec=x;      er1y_rec=y-1;
	  er2x_rec=x-1;      er2y_rec=y-1;

	  er=7;
	}

	if(flow_dir[x][y][0]==3){

	  ml=5;
	  ml1x=x-1;	  ml1y=y;
	  ml2x=x-1;        ml2y=y-1;
	  ml3x=x;	  ml3y=y-1;
	  
	  ml1x_rec=x+1;      ml1y_rec=y+1;
	  el1x_rec=x+1;      el1y_rec=y+1;
	  el2x_rec=x;      el2y_rec=y+1;

	  el=6;
	  /* no need for easy right or left, as co-ords same as ml!*/
	  
	  mr=1;
	  mr1x=x-1;	  mr1y=y;
	  mr2x=x-1;	  mr2y=y+1;
	  mr3x=x;	  mr3y=y+1;	  

	  mr1x_rec=x+1;      mr1y_rec=y-1;
	  er1x_rec=x+1;      er1y_rec=y-1;
	  er2x_rec=x;      er2y_rec=y-1;

	  er=8;
	}

	if(flow_dir[x][y][0]==4){

	  ml=6;
	  ml1x=x-1;	  ml1y=y-1;
	  ml2x=x;        ml2y=y-1;
	  ml3x=x+1;	  ml3y=y-1;
	  
	  ml1x_rec=x;      ml1y_rec=y+1;
	  el1x_rec=x;      el1y_rec=y+1;
	  el2x_rec=x-1;      el2y_rec=y+1;

	  el=7;
	  /* no need for easy right or left, as co-ords same as ml!*/
	  
	  mr=2;
	  mr1x=x-1;	  mr1y=y-1;
	  mr2x=x-1;	  mr2y=y;
	  mr3x=x-1;	  mr3y=y+1;	  

	  mr1x_rec=x+1;      mr1y_rec=y;
	  er1x_rec=x+1;      er1y_rec=y;
	  er2x_rec=x+1;      er2y_rec=y-1;

	  er=1;
	}



	if(flow_dir[x][y][0]==5){
	  
	  ml=7;
	  ml1x=x;	  ml1y=y-1;
	  ml2x=x+1;        ml2y=y-1;
	  ml3x=x+1;	  ml3y=y;
	  
	  ml1x_rec=x-1;      ml1y_rec=y+1;
	  el1x_rec=x-1;      el1y_rec=y+1;
	  el2x_rec=x-1;      el2y_rec=y;

	  el=8;
	  /* no need for easy right or left, as co-ords same as ml!*/
	  
	  mr=3;
	  mr1x=x;	  mr1y=y-1;
	  mr2x=x-1;	  mr2y=y-1;
	  mr3x=x-1;	  mr3y=y;	  

	  mr1x_rec=x+1;      mr1y_rec=y+1;
	  er1x_rec=x+1;      er1y_rec=y+1;
	  er2x_rec=x+1;    er2y_rec=y;

	  er=2;
	}

	if(flow_dir[x][y][0]==6){
	  
	  ml=8;
	  ml1x=x+1;	  ml1y=y-1;
	  ml2x=x+1;        ml2y=y;
	  ml3x=x+1;	  ml3y=y+1;
	  
	  ml1x_rec=x-1;      ml1y_rec=y;
	  el1x_rec=x-1;      el1y_rec=y;
	  el2x_rec=x-1;      el2y_rec=y-1;

	  el=1;
	  /* no need for easy right or left, as co-ords same as ml!*/
	  
	  mr=4;
	  mr1x=x+1;	  mr1y=y-1;
	  mr2x=x;	  mr2y=y-1;
	  mr3x=x-1;	  mr3y=y-1;	  

	  mr1x_rec=x;      mr1y_rec=y+1;
	  er1x_rec=x;      er1y_rec=y+1;
	  er2x_rec=x+1;      er2y_rec=y+1;

	  er=3;
	}

	if(flow_dir[x][y][0]==7){
	  
	  ml=1;
	  ml1x=x+1;	  ml1y=y;
	  ml2x=x+1;        ml2y=y+1;
	  ml3x=x;	  ml3y=y+1;
	  
	  ml1x_rec=x-1;      ml1y_rec=y-1;
	  el1x_rec=x-1;      el1y_rec=y-1;
	  el2x_rec=x;      el2y_rec=y-1;

	  el=2;
	  /* no need for easy right or left, as co-ords same as ml!*/
	  
	  mr=5;
	  mr1x=x+1;	  mr1y=y;
	  mr2x=x+1;	  mr2y=y-1;
	  mr3x=x;	  mr3y=y-1;	  

	  mr1x_rec=x-1;      mr1y_rec=y+1;
	  er1x_rec=x-1;      er1y_rec=y+1;
	  er2x_rec=x;      er2y_rec=y+1;

	  er=4;
	}

	if(flow_dir[x][y][0]==8){
	  
	  ml=2;
	  ml1x=x+1;	  ml1y=y+1;
	  ml2x=x;        ml2y=y+1;
	  ml3x=x-1;	  ml3y=y+1;
	  
	  ml1x_rec=x;      ml1y_rec=y-1;
	  el1x_rec=x;      el1y_rec=y-1;
	  el2x_rec=x+1;      el2y_rec=y-1;

	  el=3;
	  /* no need for easy right or left, as co-ords same as ml!*/
  
	  mr=6;
	  mr1x=x+1;	  mr1y=y+1;
	  mr2x=x+1;	  mr2y=y;
	  mr3x=x+1;	  mr3y=y-1;	  

	  mr1x_rec=x-1;      mr1y_rec=y;
	  er1x_rec=x-1;      er1y_rec=y;
	  er2x_rec=x-1;      er2y_rec=y+1;

	  er=5;
	}


	/* then erosion bit-- */

	if(flow_dir[x][y][1]==ml){
	  /* do medium left erosion */
	  if(elev[ml1x][ml1y]>elev[x][y]){
	    amount=(max_flow[x][y][0]/(water_depth[x][y]*DX))*2*water_depth[x][y]*time_factor*lateral_constant;
	    amount=amount/DX;
	    if((elev[ml1x][ml1y]-amount)<(bedrock[ml1x][ml1y]))amount=(elev[ml1x][ml1y]-(bedrock[ml1x][ml1y]));
	    if(amount<0)amount=0;

	    if(water_depth[ml1x_rec][ml1y_rec]>0.0001){
	      elev[ml1x_rec][ml1y_rec]+=amount;
	      elev[ml1x][ml1y]-=amount;
	      slide_GS(ml1x,ml1y,amount,ml1x_rec,ml1y_rec);
	    }
	    else{
	      elev[x][y]+=amount;
	      elev[ml1x][ml1y]-=amount;
	      slide_GS(ml1x,ml1y,amount,x,y);
	    }
	  }

	  if(elev[ml2x][ml2y]>elev[x][y]){
	    amount=(max_flow[x][y][0]/(water_depth[x][y]*DX))*2*water_depth[x][y]*time_factor*lateral_constant;
	    amount=amount/DX;
	    if((elev[ml2x][ml2y]-amount)<(bedrock[ml2x][ml2y]))amount=(elev[ml2x][ml2y]-(bedrock[ml2x][ml2y]));
	    if(amount<0)amount=0;

	    if(water_depth[ml1x_rec][ml1y_rec]>0.0001){
	      elev[ml1x_rec][ml1y_rec]+=amount;
	      elev[ml2x][ml2y]-=amount;
	      slide_GS(ml2x,ml2y,amount,ml1x_rec,ml1y_rec);
	    }
	    else{
	      elev[x][y]+=amount;
	      elev[ml2x][ml2y]-=amount;
	      slide_GS(ml2x,ml2y,amount,x,y);
	    }
	  }

	  if(elev[ml3x][ml3y]>elev[x][y]){
	    amount=(max_flow[x][y][0]/(water_depth[x][y]*DX))*2*water_depth[x][y]*time_factor*lateral_constant;
	    amount=amount/DX;
	    if((elev[ml3x][ml3y]-amount)<(bedrock[ml3x][ml3y]))amount=(elev[ml3x][ml3y]-(bedrock[ml3x][ml3y]));
	    if(amount<0)amount=0;

	    if(water_depth[ml1x_rec][ml1y_rec]>0.0001){
	      elev[ml1x_rec][ml1y_rec]+=amount;
	      elev[ml3x][ml3y]-=amount;
	      slide_GS(ml3x,ml3y,amount,ml1x_rec,ml1y_rec);
	    }
	    else{
	      elev[x][y]+=amount;
	      elev[ml3x][ml3y]-=amount;
	      slide_GS(ml3x,ml3y,amount,x,y);
	    }
	  }
	}


	if(flow_dir[x][y][1]==el){
	  /* do easy left erosion */
	  if(elev[ml1x][ml1y]>elev[x][y]){
	    amount=(max_flow[x][y][0]/(water_depth[x][y]*DX))*1*water_depth[x][y]*time_factor*lateral_constant;
	    amount=amount/DX;
	    if((elev[ml1x][ml1y]-amount)<(bedrock[ml1x][ml1y]))amount=(elev[ml1x][ml1y]-(bedrock[ml1x][ml1y]));
	    if(amount<0)amount=0;

	    if(water_depth[ml1x_rec][ml1y_rec]>0.0001){
	      elev[ml1x_rec][ml1y_rec]+=amount;
	      elev[ml1x][ml1y]-=amount;
	      slide_GS(ml1x,ml1y,amount,ml1x_rec,ml1y_rec);
	    }
	    else{
	      elev[x][y]+=amount;
	      elev[ml1x][ml1y]-=amount;
	      slide_GS(ml1x,ml1y,amount,x,y);
	    }
	  }

	  if(elev[ml2x][ml2y]>elev[x][y]){
	    amount=(max_flow[x][y][0]/(water_depth[x][y]*DX))*1*water_depth[x][y]*time_factor*lateral_constant;
	    amount=amount/DX;
	    if((elev[ml2x][ml2y]-amount)<(bedrock[ml2x][ml2y]))amount=(elev[ml2x][ml2y]-(bedrock[ml2x][ml2y]));
	    if(amount<0)amount=0;
	    if(water_depth[ml1x_rec][ml1y_rec]>0.0001){
	      elev[ml1x_rec][ml1y_rec]+=amount;
	      elev[ml2x][ml2y]-=amount;
	      slide_GS(ml2x,ml2y,amount,ml1x_rec,ml1y_rec);
	    }
	    else{
	      elev[x][y]+=amount;
	      elev[ml2x][ml2y]-=amount;
	      slide_GS(ml2x,ml2y,amount,x,y);
	    }
	  }


	  if(elev[ml3x][ml3y]>elev[x][y]){
	    amount=(max_flow[x][y][0]/(water_depth[x][y]*DX))*1*water_depth[x][y]*time_factor*lateral_constant;
	    amount=amount/DX;
	    if((elev[ml3x][ml3y]-amount)<(bedrock[ml3x][ml3y]))amount=(elev[ml3x][ml3y]-(bedrock[ml3x][ml3y]));
	    if(amount<0)amount=0;
	    if(water_depth[ml1x_rec][ml1y_rec]>0.0001){
	      elev[ml1x_rec][ml1y_rec]+=amount;
	      elev[ml3x][ml3y]-=amount;
	      slide_GS(ml3x,ml3y,amount,ml1x_rec,ml1y_rec);
	    }
	    else{
	      elev[x][y]+=amount;
	      elev[ml3x][ml3y]-=amount;
	      slide_GS(ml3x,ml3y,amount,x,y);
	    }
	  }
	}


	/****** right bend erosion ****/


	if(flow_dir[x][y][1]==mr){
	  /* do medium right erosion */
	  if(elev[mr1x][mr1y]>elev[x][y]){
	    amount=(max_flow[x][y][0]/(water_depth[x][y]*DX))*2*water_depth[x][y]*time_factor*lateral_constant;
	    amount=amount/DX;
	    if((elev[mr1x][mr1y]-amount)<(bedrock[mr1x][mr1y]))amount=(elev[mr1x][mr1y]-(bedrock[mr1x][mr1y]));
	    if(amount<0)amount=0;

	    if(water_depth[mr1x_rec][mr1y_rec]>0.0001){
	      elev[mr1x_rec][mr1y_rec]+=amount;
	      elev[mr1x][mr1y]-=amount;
	      slide_GS(mr1x,mr1y,amount,mr1x_rec,mr1y_rec);
	    }
	    else{
	      elev[x][y]+=amount;
	      elev[mr1x][mr1y]-=amount;
	      slide_GS(mr1x,mr1y,amount,x,y);
	    }
	  }

	  if(elev[mr2x][mr2y]>elev[x][y]){
	    amount=(max_flow[x][y][0]/(water_depth[x][y]*DX))*2*water_depth[x][y]*time_factor*lateral_constant;
	    amount=amount/DX;
	    if((elev[mr2x][mr2y]-amount)<(bedrock[mr2x][mr2y]))amount=(elev[mr2x][mr2y]-(bedrock[mr2x][mr2y]));
	    if(amount<0)amount=0;

	    if(water_depth[mr1x_rec][mr1y_rec]>0.0001){
	      elev[mr1x_rec][mr1y_rec]+=amount;
	      elev[mr2x][mr2y]-=amount;
	      slide_GS(mr2x,mr2y,amount,mr1x_rec,mr1y_rec);
	    }
	    else{
	      elev[x][y]+=amount;
	      elev[mr2x][mr2y]-=amount;
	      slide_GS(mr2x,mr2y,amount,x,y);
	    }
	  }

	  if(elev[mr3x][mr3y]>elev[x][y]){
	    amount=(max_flow[x][y][0]/(water_depth[x][y]*DX))*2*water_depth[x][y]*time_factor*lateral_constant;
	    amount=amount/DX;
	    if((elev[mr3x][mr3y]-amount)<(bedrock[mr3x][mr3y]))amount=(elev[mr3x][mr3y]-(bedrock[mr3x][mr3y]));
	    if(amount<0)amount=0;

	    if(water_depth[mr1x_rec][mr1y_rec]>0.0001){
	      elev[mr1x_rec][mr1y_rec]+=amount;
	      elev[mr3x][mr3y]-=amount;
	      slide_GS(mr3x,mr3y,amount,mr1x_rec,mr1y_rec);
	    }
	    else{
	      elev[x][y]+=amount;
	      elev[mr3x][mr3y]-=amount;
	      slide_GS(mr3x,mr3y,amount,x,y);
	    }
	  }
	}

	  /* do easy right erosion */	

	if(flow_dir[x][y][1]==er){

	  if(elev[mr1x][mr1y]>elev[x][y]){
	    amount=(max_flow[x][y][0]/(water_depth[x][y]*DX))*1*water_depth[x][y]*time_factor*lateral_constant;
	    amount=amount/DX;
	    if((elev[mr1x][mr1y]-amount)<(bedrock[mr1x][mr1y]))amount=(elev[mr1x][mr1y]-(bedrock[mr1x][mr1y]));
	    if(amount<0)amount=0;

	    if(water_depth[mr1x_rec][mr1y_rec]>0.0001){
	      elev[mr1x_rec][mr1y_rec]+=amount;
	      elev[mr1x][mr1y]-=amount;
	      slide_GS(mr1x,mr1y,amount,mr1x_rec,mr1y_rec);
	    }
	    else{
	      elev[x][y]+=amount;
	      elev[mr1x][mr1y]-=amount;
	      slide_GS(mr1x,mr1y,amount,x,y);
	    }

	  }

	  if(elev[mr2x][mr2y]>elev[x][y]){
	    amount=(max_flow[x][y][0]/(water_depth[x][y]*DX))*1*water_depth[x][y]*time_factor*lateral_constant;
	    amount=amount/DX;
	    if((elev[mr2x][mr2y]-amount)<(bedrock[mr2x][mr2y]))amount=(elev[mr2x][mr2y]-(bedrock[mr2x][mr2y]));
	    if(amount<0)amount=0;

	    if(water_depth[mr1x_rec][mr1y_rec]>0.0001){
	      elev[mr1x_rec][mr1y_rec]+=amount;
	      elev[mr2x][mr2y]-=amount;
	      slide_GS(mr2x,mr2y,amount,mr1x_rec,mr1y_rec);
	    }
	    else{
	      elev[x][y]+=amount;
	      elev[mr2x][mr2y]-=amount;
	      slide_GS(mr2x,mr2y,amount,x,y);
	    }

	  }

	  if(elev[mr3x][mr3y]>elev[x][y]){
	    amount=(max_flow[x][y][0]/(water_depth[x][y]*DX))*1*water_depth[x][y]*time_factor*lateral_constant;
	    amount=amount/DX;
	    if((elev[mr3x][mr3y]-amount)<(bedrock[mr3x][mr3y]))amount=(elev[mr3x][mr3y]-(bedrock[mr3x][mr3y]));
	    if(amount<0)amount=0;

	    if(water_depth[mr1x_rec][mr1y_rec]>0.0001){
	      elev[mr1x_rec][mr1y_rec]+=amount;
	      elev[mr3x][mr3y]-=amount;
	      slide_GS(mr3x,mr3y,amount,mr1x_rec,mr1y_rec);
	    }
	    else{
	      elev[x][y]+=amount;
	      elev[mr3x][mr3y]-=amount;
	      slide_GS(mr3x,mr3y,amount,x,y);
	    }
	  }
	}
      }      
    }
  }

  for(y=reach;y>=1;y--){
    inc=1;    
    while(down_scan[y][inc]>0){
      x=down_scan[y][inc];
      inc++;
      
      flow_dir[x][y][0]=0;
      flow_dir[x][y][1]=0;
      for(n=0;n<=8;n++){
	max_flow[x][y][n]=0;
      }
      if(water_depth[x][y]>0)water_depth[x][y]=0;
      
    }
  }
  
}

#if 0

void lateral()
{

  int x,y,n,inc, inctot,val;
  int hl1x,hl1y,
    hl2x,hl2y,
    hl3x,hl3y,
    hl4x,hl4y,

    ml1x,ml1y,
    ml2x,ml2y,
    ml3x,ml3y,

    el1x,el1y,
    el2x,el2y,
    el3x,el3y,

    hr1x,hr1y,
    hr2x,hr2y,
    hr3x,hr3y,
    hr4x,hr4y,

    mr1x,mr1y,
    mr2x,mr2y,
    mr3x,mr3y,

    er1x,er1y,
    er2x,er2y,
    er3x,er3y,

    hl,ml,el,hr,mr,er;

  float tot,amount;


  */

  for(y=reach;y>=1;y--){
    inc=1;    
    while(down_scan[y][inc]>0){
      x=down_scan[y][inc];
      inc++;

      if(water_depth[x][y]>0.0001&&flow_dir[x][y][0]!=0&&flow_dir[x][y][1]!=0&&(abs(flow_dir[x][y][1]-flow_dir[x][y][0])!=4)){

	/*printf("\n %d %d %f",x,y,max_flow[x][y][0]);*/

	
	hl=0;ml=0;el=0;hr=0;mr=0;er=0;

	/* now do lateral erosion bit 
	 this determines the flow direction in - */

	if(flow_dir[x][y][0]==1){

	  hl=2;
	  hl1x=x+1;	  hl1y=y+1;  
	  hl2x=x;	  hl2y=y+1;
	  hl3x=x-1;	  hl3y=y+1;
	  hl4x=x-1;	  hl4y=y;
	  
	  ml=3;
	  ml1x=x;	  ml1y=y+1;
	  ml2x=x-1;        ml2y=y+1;
	  ml3x=x-1;	  ml3y=y;
	  
	  el=4;
	  /* no need for easy right or left, as co-ords same as ml!*/

	  hr=8;
	  hr1x=x-1;	  hr1y=y+1;
	  hr2x=x;	  hr2y=y+1;
	  hr3x=x+1;	  hr3y=y+1;
	  hr4x=x+1;	  hr4y=y;
	  
	  mr=7;
	  mr1x=x;	  mr1y=y+1;
	  mr2x=x+1;	  mr2y=y+1;
	  mr3x=x+1;	  mr3y=y;	  

	  er=6;
	}

	if(flow_dir[x][y][0]==2){

	  hl=3;
	  hl1x=x;	  hl1y=y+1;  
	  hl2x=x-1;	  hl2y=y+1;
	  hl3x=x-1;	  hl3y=y;
	  hl4x=x-1;	  hl4y=y-1;
	  
	  ml=4;
	  ml1x=x-1;	  ml1y=y+1;
	  ml2x=x-1;        ml2y=y;
	  ml3x=x-1;	  ml3y=y-1;
	  
	  el=5;
	  /* no need for easy right or left, as co-ords same as ml!*/

	  hr=1;
	  hr1x=x-1;	  hr1y=y;
	  hr2x=x-1;	  hr2y=y+1;
	  hr3x=x;	  hr3y=y+1;
	  hr4x=x+1;	  hr4y=y+1;
	  
	  mr=8;
	  mr1x=x-1;	  mr1y=y+1;
	  mr2x=x;	  mr2y=y+1;
	  mr3x=x+1;	  mr3y=y+1;	  

	  er=7;
	}


	if(flow_dir[x][y][0]==3){

	  hl=4;
	  hl1x=x-1;	  hl1y=y+1;  
	  hl2x=x-1;	  hl2y=y;
	  hl3x=x-1;	  hl3y=y-1;
	  hl4x=x;	  hl4y=y-1;
	  
	  ml=5;
	  ml1x=x-1;	  ml1y=y;
	  ml2x=x-1;        ml2y=y-1;
	  ml3x=x;	  ml3y=y-1;
	  
	  el=6;
	  /* no need for easy right or left, as co-ords same as ml!*/

	  hr=2;
	  hr1x=x-1;	  hr1y=y-1;
	  hr2x=x-1;	  hr2y=y;
	  hr3x=x-1;	  hr3y=y+1;
	  hr4x=x;	  hr4y=y+1;
	  
	  mr=1;
	  mr1x=x-1;	  mr1y=y;
	  mr2x=x-1;	  mr2y=y+1;
	  mr3x=x;	  mr3y=y+1;	  

	  er=8;
	}

	if(flow_dir[x][y][0]==4){

	  hl=5;
	  hl1x=x-1;	  hl1y=y;  
	  hl2x=x-1;	  hl2y=y-1;
	  hl3x=x;	  hl3y=y-1;
	  hl4x=x+1;	  hl4y=y-1;
	  
	  ml=6;
	  ml1x=x-1;	  ml1y=y-1;
	  ml2x=x;        ml2y=y-1;
	  ml3x=x+1;	  ml3y=y-1;
	  
	  el=7;
	  /* no need for easy right or left, as co-ords same as ml!*/

	  hr=3;
	  hr1x=x;	  hr1y=y-1;
	  hr2x=x-1;	  hr2y=y-1;
	  hr3x=x-1;	  hr3y=y;
	  hr4x=x-1;	  hr4y=y+1;
	  
	  mr=2;
	  mr1x=x-1;	  mr1y=y-1;
	  mr2x=x-1;	  mr2y=y;
	  mr3x=x-1;	  mr3y=y+1;	  

	  er=1;
	}


	if(flow_dir[x][y][0]==5){

	  hl=6;
	  hl1x=x-1;	  hl1y=y-1;  
	  hl2x=x;	  hl2y=y-1;
	  hl3x=x+1;	  hl3y=y-1;
	  hl4x=x+1;	  hl4y=y;
	  
	  ml=7;
	  ml1x=x;	  ml1y=y-1;
	  ml2x=x+1;        ml2y=y-1;
	  ml3x=x+1;	  ml3y=y;
	  
	  el=8;
	  /* no need for easy right or left, as co-ords same as ml!*/

	  hr=4;
	  hr1x=x+1;	  hr1y=y-1;
	  hr2x=x;	  hr2y=y-1;
	  hr3x=x-1;	  hr3y=y-1;
	  hr4x=x-1;	  hr4y=y;
	  
	  mr=3;
	  mr1x=x;	  mr1y=y-1;
	  mr2x=x-1;	  mr2y=y-1;
	  mr3x=x-1;	  mr3y=y;	  

	  er=2;
	}

	if(flow_dir[x][y][0]==6){

	  hl=7;
	  hl1x=x;	  hl1y=y-1;  
	  hl2x=x+1;	  hl2y=y-1;
	  hl3x=x+1;	  hl3y=y;
	  hl4x=x+1;	  hl4y=y+1;
	  
	  ml=8;
	  ml1x=x+1;	  ml1y=y-1;
	  ml2x=x+1;        ml2y=y;
	  ml3x=x+1;	  ml3y=y+1;
	  
	  el=1;
	  /* no need for easy right or left, as co-ords same as ml!*/

	  hr=5;
	  hr1x=x+1;	  hr1y=y;
	  hr2x=x+1;	  hr2y=y-1;
	  hr3x=x;	  hr3y=y-1;
	  hr4x=x-1;	  hr4y=y-1;
	  
	  mr=4;
	  mr1x=x+1;	  mr1y=y-1;
	  mr2x=x;	  mr2y=y-1;
	  mr3x=x-1;	  mr3y=y-1;	  

	  er=3;
	}

	if(flow_dir[x][y][0]==7){

	  hl=8;
	  hl1x=x+1;	  hl1y=y-1;  
	  hl2x=x+1;	  hl2y=y;
	  hl3x=x+1;	  hl3y=y+1;
	  hl4x=x;	  hl4y=y+1;
	  
	  ml=1;
	  ml1x=x+1;	  ml1y=y;
	  ml2x=x+1;        ml2y=y+1;
	  ml3x=x;	  ml3y=y+1;
	  
	  el=2;
	  /* no need for easy right or left, as co-ords same as ml!*/

	  hr=6;
	  hr1x=x+1;	  hr1y=y+1;
	  hr2x=x+1;	  hr2y=y;
	  hr3x=x+1;	  hr3y=y-1;
	  hr4x=x;	  hr4y=y-1;
	  
	  mr=5;
	  mr1x=x+1;	  mr1y=y;
	  mr2x=x+1;	  mr2y=y-1;
	  mr3x=x;	  mr3y=y-1;	  

	  er=4;
	}

	if(flow_dir[x][y][0]==8){

	  hl=1;
	  hl1x=x+1;	  hl1y=y;  
	  hl2x=x+1;	  hl2y=y+1;
	  hl3x=x;	  hl3y=y+1;
	  hl4x=x-1;	  hl4y=y+1;
	  
	  ml=2;
	  ml1x=x+1;	  ml1y=y+1;
	  ml2x=x;        ml2y=y+1;
	  ml3x=x-1;	  ml3y=y+1;
	  
	  el=3;
	  /* no need for easy right or left, as co-ords same as ml!*/

	  hr=7;
	  hr1x=x;	  hr1y=y+1;
	  hr2x=x+1;	  hr2y=y+1;
	  hr3x=x+1;	  hr3y=y;
	  hr4x=x+1;	  hr4y=y-1;
	  
	  mr=6;
	  mr1x=x+1;	  mr1y=y+1;
	  mr2x=x+1;	  mr2y=y;
	  mr3x=x+1;	  mr3y=y-1;	  

	  er=5;
	}


	/* then erosion bit-- */
	
	if(flow_dir[x][y][1]==hl){
	  /* do hard left erosion */
	  if(elev[hl1x][hl1y]>elev[x][y]){
	    amount=(max_flow[x][y][0]/(water_depth[x][y]*DX))*3*water_depth[x][y]*time_factor*lateral_constant;
	    amount=amount/DX;
	    if((elev[hl1x][hl1y]-amount)<(bedrock[hl1x][hl1y]+active))amount=(elev[hl1x][hl1y]-(bedrock[hl1x][hl1y]+active));
	    if(amount<0)amount=0;
	    elev[x][y]+=amount;
	    elev[hl1x][hl1y]-=amount;
	    slide_GS(hl1x,hl1y,amount,x,y);
	  }
	  if(elev[hl2x][hl2y]>elev[x][y]){
	    amount=(max_flow[x][y][0]/(water_depth[x][y]*DX))*3*water_depth[x][y]*time_factor*lateral_constant;
	    if((elev[hl2x][hl2y]-amount)<(bedrock[hl2x][hl2y]+active))amount=(elev[hl2x][hl2y]-(bedrock[hl2x][hl2y]+active));
	    amount=amount/DX;
	    if(amount<0)amount=0;
	    elev[x][y]+=amount;
	    elev[hl2x][hl2y]-=amount;
	    slide_GS(hl2x,hl2y,amount,x,y);
	  }
	  if(elev[hl3x][hl3y]>elev[x][y]){
	    amount=(max_flow[x][y][0]/(water_depth[x][y]*DX))*3*water_depth[x][y]*time_factor*lateral_constant;
	    amount=amount/DX;
	    if((elev[hl3x][hl3y]-amount)<(bedrock[hl3x][hl3y]+active))amount=(elev[hl3x][hl3y]-(bedrock[hl3x][hl3y]+active));
	    if(amount<0)amount=0;
	    elev[x][y]+=amount;
	    elev[hl3x][hl3y]-=amount;
	    slide_GS(hl3x,hl3y,amount,x,y);
	  }
	  if(elev[hl4x][hl4y]>elev[x][y]){
	    amount=(max_flow[x][y][0]/(water_depth[x][y]*DX))*3*water_depth[x][y]*time_factor*lateral_constant;
	    amount=amount/DX;
	    if((elev[hl4x][hl4y]-amount)<(bedrock[hl4x][hl4y]+active))amount=(elev[hl4x][hl4y]-(bedrock[hl4x][hl4y]+active));
	    if(amount<0)amount=0;
	    elev[x][y]+=amount;
	    elev[hl4x][hl4y]-=amount;
	    slide_GS(hl4x,hl4y,amount,x,y);
	  }


	}
	if(flow_dir[x][y][1]==ml){
	  /* do medium left erosion */
	  if(elev[ml1x][ml1y]>elev[x][y]){
	    amount=(max_flow[x][y][0]/(water_depth[x][y]*DX))*2*water_depth[x][y]*time_factor*lateral_constant;
	    amount=amount/DX;
	    if((elev[ml1x][ml1y]-amount)<(bedrock[ml1x][ml1y]+active))amount=(elev[ml1x][ml1y]-(bedrock[ml1x][ml1y]+active));
	    if(amount<0)amount=0;
	    elev[x][y]+=amount;
	    elev[ml1x][ml1y]-=amount;
	    slide_GS(ml1x,ml1y,amount,x,y);
	  }
	  if(elev[ml2x][ml2y]>elev[x][y]){
	    amount=(max_flow[x][y][0]/(water_depth[x][y]*DX))*2*water_depth[x][y]*time_factor*lateral_constant;
	    amount=amount/DX;
	    if((elev[ml2x][ml2y]-amount)<(bedrock[ml2x][ml2y]+active))amount=(elev[ml2x][ml2y]-(bedrock[ml2x][ml2y]+active));
	    if(amount<0)amount=0;
	    elev[x][y]+=amount;
	    elev[ml2x][ml2y]-=amount;
	    slide_GS(ml2x,ml2y,amount,x,y);
	  }
	  if(elev[ml3x][ml3y]>elev[x][y]){
	    amount=(max_flow[x][y][0]/(water_depth[x][y]*DX))*2*water_depth[x][y]*time_factor*lateral_constant;
	    amount=amount/DX;
	    if((elev[ml3x][ml3y]-amount)<(bedrock[ml3x][ml3y]+active))amount=(elev[ml3x][ml3y]-(bedrock[ml3x][ml3y]+active));
	    if(amount<0)amount=0;
	    elev[x][y]+=amount;
	    elev[ml3x][ml3y]-=amount;
	    slide_GS(ml3x,ml3y,amount,x,y);
	  }
	}

	if(flow_dir[x][y][1]==el){
	  /* do easy left erosion */
	  if(elev[ml1x][ml1y]>elev[x][y]){
	    amount=(max_flow[x][y][0]/(water_depth[x][y]*DX))*1*water_depth[x][y]*time_factor*lateral_constant;
	    amount=amount/DX;
	    if((elev[ml1x][ml1y]-amount)<(bedrock[ml1x][ml1y]+active))amount=(elev[ml1x][ml1y]-(bedrock[ml1x][ml1y]+active));
	    if(amount<0)amount=0;
	    elev[x][y]+=amount;
	    elev[ml1x][ml1y]-=amount;
	    slide_GS(ml1x,ml1y,amount,x,y);
	  }
	  if(elev[ml2x][ml2y]>elev[x][y]){
	    amount=(max_flow[x][y][0]/(water_depth[x][y]*DX))*1*water_depth[x][y]*time_factor*lateral_constant;
	    amount=amount/DX;
	    if((elev[ml2x][ml2y]-amount)<(bedrock[ml2x][ml2y]+active))amount=(elev[ml2x][ml2y]-(bedrock[ml2x][ml2y]+active));
	    if(amount<0)amount=0;
	    elev[x][y]+=amount;
	    elev[ml2x][ml2y]-=amount;
	    slide_GS(ml2x,ml2y,amount,x,y);
	  }
	  if(elev[ml3x][ml3y]>elev[x][y]){
	    amount=(max_flow[x][y][0]/(water_depth[x][y]*DX))*1*water_depth[x][y]*time_factor*lateral_constant;
	    amount=amount/DX;
	    if((elev[ml3x][ml3y]-amount)<(bedrock[ml3x][ml3y]+active))amount=(elev[ml3x][ml3y]-(bedrock[ml3x][ml3y]+active));
	    if(amount<0)amount=0;
	    elev[x][y]+=amount;
	    elev[ml3x][ml3y]-=amount;
	    slide_GS(ml3x,ml3y,amount,x,y);
	  }
	}

	if(flow_dir[x][y][1]==hr){
	  /* do hard right erosion */
	  if(elev[hr1x][hr1y]>elev[x][y]){
	    amount=(max_flow[x][y][0]/(water_depth[x][y]*DX))*3*water_depth[x][y]*time_factor*lateral_constant;
	    amount=amount/DX;
	    if((elev[hr1x][hr1y]-amount)<(bedrock[hr1x][hr1y]+active))amount=(elev[hr1x][hr1y]-(bedrock[hr1x][hr1y]+active));
	    if(amount<0)amount=0;
	    elev[x][y]+=amount;
	    elev[hr1x][hr1y]-=amount;
	    slide_GS(hr1x,hr1y,amount,x,y);
	  }
	  if(elev[hr2x][hr2y]>elev[x][y]){
	    amount=(max_flow[x][y][0]/(water_depth[x][y]*DX))*3*water_depth[x][y]*time_factor*lateral_constant;
	    amount=amount/DX;
	    if((elev[hr2x][hr2y]-amount)<(bedrock[hr2x][hr2y]+active))amount=(elev[hr2x][hr2y]-(bedrock[hr2x][hr2y]+active));
	    if(amount<0)amount=0;
	    elev[x][y]+=amount;
	    elev[hr2x][hr2y]-=amount;
	    slide_GS(hr2x,hr2y,amount,x,y);
	  }
	  if(elev[hr3x][hr3y]>elev[x][y]){
	    amount=(max_flow[x][y][0]/(water_depth[x][y]*DX))*3*water_depth[x][y]*time_factor*lateral_constant;
	    amount=amount/DX;
	    if((elev[hr3x][hr3y]-amount)<(bedrock[hr3x][hr3y]+active))amount=(elev[hr3x][hr3y]-(bedrock[hr3x][hr3y]+active));
	    if(amount<0)amount=0;
	    elev[x][y]+=amount;
	    elev[hr3x][hr3y]-=amount;
	    slide_GS(hr3x,hr3y,amount,x,y);
	  }
	  if(elev[hr4x][hr4y]>elev[x][y]){
	    amount=(max_flow[x][y][0]/(water_depth[x][y]*DX))*3*water_depth[x][y]*time_factor*lateral_constant;
	    amount=amount/DX;
	    if((elev[hr4x][hr4y]-amount)<(bedrock[hr4x][hr4y]+active))amount=(elev[hr4x][hr4y]-(bedrock[hr4x][hr4y]+active));
	    if(amount<0)amount=0;
	    elev[x][y]+=amount;
	    elev[hr4x][hr4y]-=amount;
	    slide_GS(hl4x,hl4y,amount,x,y);
	  }
	}
	if(flow_dir[x][y][1]==mr){
	  /* do medium right erosion */
	  if(elev[mr1x][mr1y]>elev[x][y]){
	    amount=(max_flow[x][y][0]/(water_depth[x][y]*DX))*2*water_depth[x][y]*time_factor*lateral_constant;
	    amount=amount/DX;
	    if((elev[mr1x][mr1y]-amount)<(bedrock[mr1x][mr1y]+active))amount=(elev[mr1x][mr1y]-(bedrock[mr1x][mr1y]+active));
	    if(amount<0)amount=0;
	    elev[x][y]+=amount;
	    elev[mr1x][mr1y]-=amount;
	    slide_GS(mr1x,mr1y,amount,x,y);
	  }
	  if(elev[mr2x][mr2y]>elev[x][y]){
	    amount=(max_flow[x][y][0]/(water_depth[x][y]*DX))*2*water_depth[x][y]*time_factor*lateral_constant;
	    amount=amount/DX;
	    if((elev[mr2x][mr2y]-amount)<(bedrock[mr2x][mr2y]+active))amount=(elev[mr2x][mr2y]-(bedrock[mr2x][mr2y]+active));
	    if(amount<0)amount=0;
	    elev[x][y]+=amount;
	    elev[mr2x][mr2y]-=amount;
	    slide_GS(mr2x,mr2y,amount,x,y);
	  }
	  if(elev[mr3x][mr3y]>elev[x][y]){
	    amount=(max_flow[x][y][0]/(water_depth[x][y]*DX))*2*water_depth[x][y]*time_factor*lateral_constant;
	    amount=amount/DX;
	    if((elev[mr3x][mr3y]-amount)<(bedrock[mr3x][mr3y]+active))amount=(elev[mr3x][mr3y]-(bedrock[mr3x][mr3y]+active));
	    if(amount<0)amount=0;
	    elev[x][y]+=amount;
	    elev[mr3x][mr3y]-=amount;
	    slide_GS(mr3x,mr3y,amount,x,y);
	  }
	}
	if(flow_dir[x][y][1]==er){
	  /* do easy right erosion */
	  if(elev[mr1x][mr1y]>elev[x][y]){
	    amount=(max_flow[x][y][0]/(water_depth[x][y]*DX))*1*water_depth[x][y]*time_factor*lateral_constant;
	    amount=amount/DX;
	    if((elev[mr1x][mr1y]-amount)<(bedrock[mr1x][mr1y]+active))amount=(elev[mr1x][mr1y]-(bedrock[mr1x][mr1y]+active));
	    if(amount<0)amount=0;
	    elev[x][y]+=amount;
	    elev[mr1x][mr1y]-=amount;
	    slide_GS(mr1x,mr1y,amount,x,y);
	  }
	  if(elev[mr2x][mr2y]>elev[x][y]){
	    amount=(max_flow[x][y][0]/(water_depth[x][y]*DX))*1*water_depth[x][y]*time_factor*lateral_constant;
	    amount=amount/DX;
	    if((elev[mr2x][mr2y]-amount)<(bedrock[mr2x][mr2y]+active))amount=(elev[mr2x][mr2y]-(bedrock[mr2x][mr2y]+active));
	    if(amount<0)amount=0;
	    elev[x][y]+=amount;
	    elev[mr2x][mr2y]-=amount;
	    slide_GS(mr2x,mr2y,amount,x,y);
	  }
	  if(elev[mr3x][mr3y]>elev[x][y]){
	    amount=(max_flow[x][y][0]/(water_depth[x][y]*DX))*1*water_depth[x][y]*time_factor*lateral_constant;
	    amount=amount/DX;
	    if((elev[mr3x][mr3y]-amount)<(bedrock[mr3x][mr3y]+active))amount=(elev[mr3x][mr3y]-(bedrock[mr3x][mr3y]+active));
	    if(amount<0)amount=0;
	    elev[x][y]+=amount;
	    elev[mr3x][mr3y]-=amount;
	    slide_GS(mr3x,mr3y,amount,x,y);
	  }
	}
	
      }

    }
  }


      water_depth[x][y]=0;
      flow_dir[x][y][0]=0;
      flow_dir[x][y][1]=0;
      for(n=0;n<=8;n++)max_flow[x][y][n]=0;
  
}

#endif


/******************** add new grainsize cells ************************************/

void addGS(x,y)
int x;
int y;
{

	int n,z;

	grain_array_tot++;
	index[x][y]=grain_array_tot;
	for(n=0;n<=G_MAX;n++)temp[grain_array_tot][n][0]=0;

	z=0;
	/*if(x>203&&x<430&&y>24&&y<101)z=1;*/

	
	grain[grain_array_tot][0][z]=1*active; 	
	grain[grain_array_tot][1][z]=0;
	grain[grain_array_tot][2][z]=0;
	grain[grain_array_tot][3][z]=0;
	grain[grain_array_tot][4][z]=0;
	grain[grain_array_tot][5][z]=0;
	grain[grain_array_tot][6][z]=0;	
	grain[grain_array_tot][7][z]=0;
	grain[grain_array_tot][8][z]=0;
	grain[grain_array_tot][9][z]=0;
	grain[grain_array_tot][10][z]=0;
	
	/*
	grain[grain_array_tot][0][z]=0; 	
	grain[grain_array_tot][1][z]=active*0.05;
	grain[grain_array_tot][2][z]=active*0.15;
	grain[grain_array_tot][3][z]=active*0.275;
	grain[grain_array_tot][4][z]=active*0.25;
	grain[grain_array_tot][5][z]=active*0.1;
	grain[grain_array_tot][6][z]=active*0.075;
	grain[grain_array_tot][7][z]=active*0.05;
	grain[grain_array_tot][8][z]=active*0.025;
	grain[grain_array_tot][9][z]=active*0.025;
	grain[grain_array_tot][10][z]=0;
	*/	

	for(n=0;n<=9;n++){
		strata[grain_array_tot][n][0][z]=(active)*0.05;
		strata[grain_array_tot][n][1][z]=(active)*0.15;
		strata[grain_array_tot][n][2][z]=(active)*0.275;
		strata[grain_array_tot][n][3][z]=(active)*0.25;
		strata[grain_array_tot][n][4][z]=(active)*0.1;
		strata[grain_array_tot][n][5][z]=(active)*0.075;
		strata[grain_array_tot][n][6][z]=(active)*0.05;
		strata[grain_array_tot][n][7][z]=(active)*0.025;
		strata[grain_array_tot][n][8][z]=(active)*0.025;
	}
	
       	sort_active(x,y);


}



/******************* third slide routine *****************************************/


void slide_3()
{
	int x,y,inc;
	float fail_angle=60;
	float wet_factor;
	float factor=tan((fail_angle*(3.141592654/180)))*DX;
	float diff=0;

	for(y=1;y<reach;y++){
		inc=1;
		while(down_scan[y][inc]>0){
		  x=down_scan[y][inc];
		  inc++;	
		        water_depth[x][y]=0;
			/** check to see if under water **/
			wet_factor=factor;
			if(water_depth[x][y]>0.01)wet_factor=factor/2;
			if(elev[x][y]<=(bedrock[x][y]+active))wet_factor=10*DX;

				/** chexk landslides in channel slowly */

				if(((elev[x][y]-elev[x+1][y+1])/1.41)>wet_factor&&elev[x+1][y+1]>0&&water_depth[x+1][y+1]!=-9999){
				        diff=((elev[x][y]-elev[x+1][y+1])/1.41)-wet_factor;
					if((elev[x][y]-diff)<(bedrock[x][y]+active))diff=(elev[x][y]-(bedrock[x][y]+active));
					elev[x][y]-=diff;
					elev[x+1][y+1]+=diff;
					slide_GS(x,y,diff,x+1,y+1);
				}
				if((elev[x][y]-elev[x][y+1])>wet_factor&&elev[x][y+1]>0&&water_depth[x][y+1]!=-9999){
				        diff=(elev[x][y]-elev[x][y+1])-wet_factor;
					if((elev[x][y]-diff)<(bedrock[x][y]+active))diff=(elev[x][y]-(bedrock[x][y]+active));
					elev[x][y]-=diff;
					elev[x][y+1]+=diff;
					slide_GS(x,y,diff,x,y+1);
				}
				if(((elev[x][y]-elev[x-1][y+1])/1.41)>wet_factor&&elev[x-1][y+1]>0&&water_depth[x-1][y+1]!=-9999){
				        diff=((elev[x][y]-elev[x-1][y+1])/1.41)-wet_factor;
					if((elev[x][y]-diff)<(bedrock[x][y]+active))diff=(elev[x][y]-(bedrock[x][y]+active));
					elev[x][y]-=diff;
					elev[x-1][y+1]+=diff;
					slide_GS(x,y,diff,x-1,y+1);
				}
				if((elev[x][y]-elev[x-1][y])>wet_factor&&elev[x-1][y]>0&&water_depth[x-1][y]!=-9999){
				        diff=(elev[x][y]-elev[x-1][y])-wet_factor;
					if((elev[x][y]-diff)<(bedrock[x][y]+active))diff=(elev[x][y]-(bedrock[x][y]+active));
					elev[x][y]-=diff;
					elev[x-1][y]+=diff;
					slide_GS(x,y,diff,x-1,y);
				}	

				if(((elev[x][y]-elev[x-1][y-1])/1.41)>wet_factor&&elev[x-1][y-1]>0&&water_depth[x-1][y-1]!=-9999){
				        diff=((elev[x][y]-elev[x-1][y-1])/1.41)-wet_factor;
					if((elev[x][y]-diff)<(bedrock[x][y]+active))diff=(elev[x][y]-(bedrock[x][y]+active));
					elev[x][y]-=diff;
					elev[x-1][y-1]+=diff;
					slide_GS(x,y,diff,x-1,y-1);
				}
				if((elev[x][y]-elev[x][y-1])>wet_factor&&elev[x][y-1]>0&&water_depth[x][y-1]!=-9999){
				        diff=(elev[x][y]-elev[x][y-1])-wet_factor;
					if((elev[x][y]-diff)<(bedrock[x][y]+active))diff=(elev[x][y]-(bedrock[x][y]+active));
					elev[x][y]-=diff;
					elev[x][y-1]+=diff;
					slide_GS(x,y,diff,x,y-1);
				}
				if(((elev[x][y]-elev[x+1][y-1])/1.41)>wet_factor&&elev[x+1][y-1]>0&&water_depth[x+1][y-1]!=-9999){
				        diff=((elev[x][y]-elev[x+1][y-1])/1.41)-wet_factor;
					if((elev[x][y]-diff)<(bedrock[x][y]+active))diff=(elev[x][y]-(bedrock[x][y]+active));
					elev[x][y]-=diff;
					elev[x+1][y-1]+=diff;
					slide_GS(x,y,diff,x+1,y-1);
				}


	      /* if(dist[n][fract]<0.000001)dist[n][fract]=0;*/	if((elev[x][y]-elev[x+1][y])>wet_factor&&elev[x+1][y]>0&&water_depth[x+1][y]!=-9999){
				        diff=(elev[x][y]-elev[x+1][y])-wet_factor;
					if((elev[x][y]-diff)<(bedrock[x][y]+active))diff=(elev[x][y]-(bedrock[x][y]+active));
					elev[x][y]-=diff;
					elev[x+1][y]+=diff;
					slide_GS(x,y,diff,x+1,y);
				}
			
		}
	}

}

/*************** find active layer depth 2 times d50*****************************/
/* have to remeber to change active layer quantities !!                         */

void find_active(x,y)
int x;
int y;
{
}


/*************************** sort out active layer depth **************/
void sort_active(x,y)
int x;
int y;
{

        int temp_centre;
	float sub_total=0;
	float total=0;
	float total2=0;
	float amount=0;
	float amount2=0;
	float sub_tot;
	int n,z,q,c;
	
	/** for variable active layer **/
	/*active=find_active(x,y);*/

	/** for fixed ie d90 ****/

	temp_centre=CENTRE;
	total=0;

	for(n=0;n<=G_MAX;n++){
	  for(q=0;q<=(TRACERS-1);q++){
		if(grain[temp_centre][n][q]<0){
			printf("SHITE !! %d\n",q);
			  grain[temp_centre][n][q]=0;
		}
	      
		total+=grain[temp_centre][n][q];
	  }
	}
	
	/** if depositing.... */

	if(total>(active)){
		
		/* first deal with top layer */
		amount=(total-active);

		if(grain[temp_centre][G_MAX][0]>0){
		  if(grain[temp_centre][G_MAX][0]>=amount){
		   grain[temp_centre][G_MAX][0]-=amount;
		   amount=0;
		  }
		  else{
		   amount-=grain[temp_centre][G_MAX][0];
		   grain[temp_centre][G_MAX][0]=0;
		  }
		}


		for(n=0;n<=(G_MAX-1);n++){
			if(grain_tot(temp_centre,n)>0){
				amount2=(grain_tot(temp_centre,n)/total)*amount;
			    	if(n>0&&n<G_MAX){
				  sub_tot=grain_tot(temp_centre,n);
				  if(sub_tot>0){
	      	      		   for(q=0;q<=(TRACERS-1);q++){
				     strata[temp_centre][0][n-1][q]+=amount2*(grain[temp_centre][n][q]/sub_tot);
				   }
				  }
				}
				sub_tot=grain_tot(temp_centre,n);
				if(sub_tot>0){
				  for(q=0;q<=(TRACERS-1);q++){
				    grain[temp_centre][n][q]-=amount2*(grain[temp_centre][n][q]/sub_tot);
				  }
				}
			}
		}
		       	
			/* then workk down through the strata **/

		for(z=0;z<=9;z++){
			total2=0;
			for(n=0;n<=(G_MAX-2);n++)total2+=strata_tot(temp_centre,z,n);
			amount=(total2-(active));
			for(n=0;n<=(G_MAX-2);n++){
				if(strata_tot(temp_centre,z,n)>0&&amount>0){
				  if(z<9){
				    sub_tot=strata_tot(temp_centre,z,n);
				    for(q=0;q<=(TRACERS-1);q++){
				      strata[temp_centre][z+1][n][q]+=(strata[temp_centre][z][n][q]/sub_tot)*((sub_tot/total2)*amount);
				    }
				  }
				  sub_tot=strata_tot(temp_centre,z,n);
				  for(q=0;q<=(TRACERS-1);q++){
					strata[temp_centre][z][n][q]-=(strata[temp_centre][z][n][q]/sub_tot)*((sub_tot/total2)*amount);
				  }
				}	
			}		
		}


	}

	total=0;
	for(n=0;n<=G_MAX;n++){
	  for(q=0;q<=(TRACERS-1);q++){
		total+=grain[temp_centre][n][q];
		if(grain[temp_centre][n][q]<0)printf("First PANTS!!\n");
	  }
	}

	/*** if eroding ... **/

	if(total<(active)){
		amount=(active-total);
	
       		if(elev[x][y]<=(bedrock[x][y]+active)){
		  grain[temp_centre][G_MAX][0]+=amount;
		}

		else{
			total2=0;
			for(n=0;n<=(G_MAX-2);n++){
				if(strata_tot(temp_centre,0,n)<0)printf("opps 1\n");
				/*if(strata[temp_centre][0][n]<0)strata[CENTRE][0][n]=0;*/
				total2+=strata_tot(temp_centre,0,n);
			}

			for(n=0;n<=(G_MAX-2);n++){
				if(strata_tot(temp_centre,0,n)>0){
					amount2=((strata_tot(temp_centre,0,n)/total2)*amount);
					sub_tot=strata_tot(temp_centre,0,n);
					if(sub_tot>0){
					for(q=0;q<=(TRACERS-1);q++){
					  grain[temp_centre][n+1][q]+=amount2*(strata[temp_centre][0][n][q]/sub_tot);
					  strata[temp_centre][0][n][q]-=amount2*(strata[temp_centre][0][n][q]/sub_tot);
					}
					}
				}
			}
		
			for(z=0;z<=8;z++){
			
				total=0;
				for(n=0;n<=(G_MAX-2);n++){
				  for(q=0;q<=(TRACERS-1);q++){
				    if(strata[temp_centre][z][n][q]<0)strata[temp_centre][z][n][q]=0;
				  }
				  total+=strata_tot(temp_centre,z,n);
				}

				amount=(active)-total;

				total2=0;
				for(n=0;n<=(G_MAX-2);n++){
				  for(q=0;q<=(TRACERS-1);q++){
				    if(strata[temp_centre][z+1][n][q]<0)strata[temp_centre][z+1][n][q]=0;
				  }
				  total2+=strata_tot(temp_centre,z+1,n);
				}

				for(n=0;n<=(G_MAX-2);n++){
					if(strata_tot(temp_centre,z+1,n)>0&&amount>0){
					  sub_tot=strata_tot(temp_centre,z+1,n);
					  if(sub_tot>0){
					  amount2=(sub_tot/total2)*amount;
					  for(q=0;q<=(TRACERS-1);q++){
						strata[temp_centre][z][n][q]+=(strata[temp_centre][z+1][n][q]/sub_tot)*amount2;
						strata[temp_centre][z+1][n][q]-=(strata[temp_centre][z+1][n][q]/sub_tot)*amount2;
					  }
					  }
					}
					
				}
			}
			c=0;
			/*if(x>203&&x<430&&y>24&&y<101)c=1;*/		
			strata[temp_centre][9][0][c]+=amount*0.05;
			strata[temp_centre][9][1][c]+=amount*0.15;
			strata[temp_centre][9][2][c]+=amount*0.275;
			strata[temp_centre][9][3][c]+=amount*0.25;
			strata[temp_centre][9][4][c]+=amount*0.1;
			strata[temp_centre][9][5][c]+=amount*0.075;
			strata[temp_centre][9][6][c]+=amount*0.05;
			strata[temp_centre][9][7][c]+=amount*0.025;
			strata[temp_centre][9][8][c]+=amount*0.025;
		}
	}
		
	total=0;
	for(n=0;n<=G_MAX;n++){
	  for(q=0;q<=(TRACERS-1);q++){
		if(grain[temp_centre][n][q]<0)printf("PANTS!! %f %d %d %d\n",grain[temp_centre][n][q],q,x,y);
		if(grain[temp_centre][n][q]<0)grain[temp_centre][n][q]=0;
		total+=grain[temp_centre][n][q];
	  }
	}


	if(total>active){

		for(n=0;n<=G_MAX;n++){
		  for(q=0;q<=(TRACERS-1);q++){
			if(grain[temp_centre][n][q]>(total-active)){
				grain[temp_centre][n][q]-=(total-active);
				total=active;
			}
		  }
		}
	}

	if(total<active){

       		if(elev[x][y]<=(bedrock[x][y]+active)){
			grain[temp_centre][G_MAX][0]+=(active-total);
		}
		else{
			grain[temp_centre][3][0]+=(active-total);
		}
	}

	total=0;
	for(n=0;n<=G_MAX;n++){
	  for(q=0;q<=(TRACERS-1);q++){
		total+=grain[temp_centre][n][q];
		if(grain[temp_centre][n][q]<0)printf("More PANTS!!\n");
	  }
	}

}

/*******************************************/

float four_dp(inval)
float inval;
{
	int a,b;
	float c,outval;

	c=(inval*100000000);
	a=c;
	outval=a;
	outval=outval/100000000;
	return outval;
}	


/*******************************  grainsize erosion !! ***************/



void erode2(p,t,depth,slope)
int p;
float t;
float depth;
float slope;
{
	/* here, forme erode, the x,y co-ords of the cell to be eroded from are passed.
	along with the direction to be eroded to, xinc, yinc, as wellas the pow er (t)
	I've set the active layer, as a fixed size, (2*d84) or for this case approx
	50cm. This aslways remains fixed (depth) and if depositing, any excess is moved down,
	if eroding moved up into active layer. updating etc is going to be complex. can assume,
	that as program will only allow 10% of depth eroded, 50cm means 5m deep flow, will probably never
	happen */

	/* this const, muller is Richards def, not other which is 0.4063 */

	float v;
	float muller=0.2593;
	float manning=0.03;

	float tcrit1=4;
	float tcrit2=8;
	float tcrit3=16;
	float tcrit4=32;
	float tcrit5=64;

	float d0=0.27;
	float d1=0.001;
	float d2=0.002;
	float d3=0.004;
	float d4=0.008;
	float d5=0.016;
	float d6=0.032;
	float d7=0.064;
	float d8=0.128;
	float d9=0.256;
	float d10=500000;

	float amount=0;
	float amt0=0;
	float amt1=0;
	float amt2=0;
	float amt3=0;
	float amt4=0;
	float amt5=0;
	float amt6=0;
	float amt7=0;
	float amt8=0;
	float amt9=0;
	float amt10=0;

	float ibr=0.01;
	float Wr=0.5;
	float Yr=0.1;
	float Dr=0.0011;

	float wo_constant=50000; /** wrong!!*/

	float w=0;
	float wo1=0;
	float wo2=0;
	float wo3=0;
	float wo4=0;
	float wo5=0;

	float devils_fork0=0;
	float devils_fork1=0;
	float devils_fork2=0;
	float devils_fork3=0;
	float devils_fork4=0;
	float devils_fork5=0;
	float devils_fork6=0;
	float devils_fork7=0;
	float devils_fork8=0;
	float devils_fork9=0;
	float devils_fork10=0;

	float omega0=0;
	float omega1=0;
	float omega2=0;
	float omega3=0;
	float omega4=0;
	float omega5=0;
	float omega6=0;
	float omega7=0;
	float omega8=0;
	float omega9=0;
	float omega10=0;

	int n;


	v=(pow(depth,0.67)*pow(slope,0.5))/manning;

	/*** using Meyer Peter & Muller */
	/*

	if(t>tcrit5)amt5=(muller*pow((t-tcrit5),1.5));
	if(t>tcrit4)amt4=(muller*pow((t-tcrit4),1.5));
	if(t>tcrit3)amt3=(muller*pow((t-tcrit3),1.5));
	if(t>tcrit2)amt2=(muller*pow((t-tcrit2),1.5));
	if(t>tcrit1)amt1=(muller*pow((t-tcrit1),1.5));

	*/

	/*** Using bagnold 1980 **************/
	/* assuming density of sed = 2250 & water 1000 */
	/* this version gives weight, so divide by density 2250 to give volume */
	/*

	w=t*v;  
	wo1=wo_constant*pow(d1,1.5)*log((12*depth/d1));
	wo2=wo_constant*pow(d2,1.5)*log((12*depth/d2));	
	wo3=wo_constant*pow(d3,1.5)*log((12*depth/d3));
	wo4=wo_constant*pow(d4,1.5)*log((12*depth/d4));
	wo5=wo_constant*pow(d5,1.5)*log((12*depth/d5));


	if(w>wo1)amt1=(1.8*0.5*pow(((w-wo1)/Wr),1.5)*pow((depth/Yr),-0.6667)*pow((d1/Dr),-0.5))/2250;
	if(w>wo2)amt2=(1.8*0.5*pow(((w-wo2)/Wr),1.5)*pow((depth/Yr),-0.6667)*pow((d2/Dr),-0.5))/2250;
	if(w>wo3)amt3=(1.8*0.5*pow(((w-wo3)/Wr),1.5)*pow((depth/Yr),-0.6667)*pow((d3/Dr),-0.5))/2250;
	if(w>wo4)amt4=(1.8*0.5*pow(((w-wo4)/Wr),1.5)*pow((depth/Yr),-0.6667)*pow((d4/Dr),-0.5))/2250;
	if(w>wo5)amt5=(1.8*0.5*pow(((w-wo5)/Wr),1.5)*pow((depth/Yr),-0.6667)*pow((d5/Dr),-0.5))/2250;

	*/

	/********* using einstein brown 1950 *********/
	/* assuming depth = R */
	/* all taken from steves fluid book */

	devils_fork0=((2250-1000)*d0)/(1000*depth*slope);	
	devils_fork1=((2250-1000)*d1)/(1000*depth*slope);
	devils_fork2=((2250-1000)*d2)/(1000*depth*slope);	
	devils_fork3=((2250-1000)*d3)/(1000*depth*slope);
	devils_fork4=((2250-1000)*d4)/(1000*depth*slope);
	devils_fork5=((2250-1000)*d5)/(1000*depth*slope);
	devils_fork6=((2250-1000)*d6)/(1000*depth*slope);
	devils_fork7=((2250-1000)*d7)/(1000*depth*slope);
	devils_fork8=((2250-1000)*d8)/(1000*depth*slope);
	devils_fork9=((2250-1000)*d9)/(1000*depth*slope);
	devils_fork10=((2250-1000)*d10)/(1000*depth*slope);

	omega0=40*pow((1/devils_fork0),3);
	omega1=40*pow((1/devils_fork1),3);
	omega2=40*pow((1/devils_fork2),3);    
	omega3=40*pow((1/devils_fork3),3);
	omega4=40*pow((1/devils_fork4),3);
	omega5=40*pow((1/devils_fork5),3);
	omega6=40*pow((1/devils_fork6),3);
	omega7=40*pow((1/devils_fork7),3);
	omega8=40*pow((1/devils_fork8),3);
	omega9=40*pow((1/devils_fork9),3);
	omega10=40*pow((1/devils_fork10),3);

	amt0=omega0/sqrt(1000/((2250-1000)*9.8*pow(d0,3)));	
	amt1=omega1/sqrt(1000/((2250-1000)*9.8*pow(d1,3)));
	amt2=omega2/sqrt(1000/((2250-1000)*9.8*pow(d2,3)));
	amt3=omega3/sqrt(1000/((2250-1000)*9.8*pow(d3,3)));
	amt4=omega4/sqrt(1000/((2250-1000)*9.8*pow(d4,3)));
	amt5=omega5/sqrt(1000/((2250-1000)*9.8*pow(d5,3)));
	amt6=omega6/sqrt(1000/((2250-1000)*9.8*pow(d6,3)));
	amt7=omega7/sqrt(1000/((2250-1000)*9.8*pow(d7,3)));
	amt8=omega8/sqrt(1000/((2250-1000)*9.8*pow(d8,3)));
	amt9=omega9/sqrt(1000/((2250-1000)*9.8*pow(d9,3)));
	amt10=omega10/sqrt(1000/((2250-1000)*9.8*pow(d10,3)));

	dist[p][0]=amt0*time_factor;
	dist[p][1]=amt1*time_factor;
	dist[p][2]=amt2*time_factor;
	dist[p][3]=amt3*time_factor;
	dist[p][4]=amt4*time_factor;
	dist[p][5]=amt5*time_factor;
	dist[p][6]=amt6*time_factor;
	dist[p][7]=amt7*time_factor;
	dist[p][8]=amt8*time_factor;
	dist[p][9]=amt9*time_factor;
	dist[p][10]=0; /*amt10*time_factor;*/

}



/**********************************************************************/


void erode3(fract, grid_val,x,y)
int fract;
int grid_val;
int x;
int y;
{

        int n,temp_centre;
	float tempb=0;
	float tempamt=0;
	double total=0;
	double total2=0;

	temp_centre=CENTRE;
	if(index[x][y]==-9999)addGS(x,y);
	
	for(n=1;n<=8;n++){
	  total+=dist[n][fract];
	}

	/******* lines like the next one, are assential to ensure that it adds new eroded sqaures to the list. **/
	if(grain_tot(temp_centre,fract)>0.00001){


	  if(index[x][y]==-9999)addGS(x,y);
      
	  if(total>=grain_tot(temp_centre,fract)){
	    total2=0;
	    for(n=1;n<=8;n++){
	      dist[n][fract]=dist[n][fract]*(grain_tot(temp_centre,fract)/total)*0.99999;
	      total2+=dist[n][fract];
	    }


	  }
	       		if(dist[1][fract]>0.0000){
				if(index[x-1][y-1]==-9999)addGS(x-1,y-1);
				erode4(temp_centre,fract,dist[1][fract],NW);
					
				/*				if(grain[temp_centre][fract]+temp[temp_centre][fract]<0){
				  	printf("one %f %f %d %d\n",grain[temp_centre][fract],temp[temp_centre][fract],x,y);
				        temp[NW][fract]-=fabs(grain[temp_centre][fract]+temp[temp_centre][fract]);
					temp[temp_centre][fract]+=fabs(grain[temp_centre][fract]+temp[temp_centre][fract]);
					}*/
			}
			if(dist[2][fract]>0.0000){
				if(index[x][y-1]==-9999)addGS(x,y-1);
				erode4(temp_centre,fract,dist[2][fract],N);
				
				/*				if(grain[temp_centre][fract]+temp[temp_centre][fract]<0){
				  printf("two %f %f %d %d\n",grain[temp_centre][fract],temp[temp_centre][fract],x,y);
					temp[N][fract]-=fabs(grain[temp_centre][fract]+temp[temp_centre][fract]);
					temp[temp_centre][fract]+=fabs(grain[temp_centre][fract]+temp[temp_centre][fract]);
					}*/

			}
			if(dist[3][fract]>0.0000){
				if(index[x+1][y-1]==-9999)addGS(x+1,y-1);
				erode4(temp_centre,fract,dist[3][fract],NE);

				
				/*				if(grain[temp_centre][fract]+temp[temp_centre][fract]<0){
				  	printf("three %f %f %d %d\n",grain[temp_centre][fract],temp[temp_centre][fract],x,y);
					temp[NE][fract]-=fabs(grain[temp_centre][fract]+temp[temp_centre][fract]);
					temp[temp_centre][fract]+=fabs(grain[temp_centre][fract]+temp[temp_centre][fract]);
					}*/
			}
			if(dist[4][fract]>0.0000){
				if(index[x+1][y]==-9999)addGS(x+1,y);
				erode4(temp_centre,fract,dist[4][fract],E);
				
				/*			       	if(grain[temp_centre][fract]+temp[temp_centre][fract]<0){
				  printf("four %f %f %d %d\n",grain[temp_centre][fract],temp[temp_centre][fract],x,y);
					temp[E][fract]-=fabs(grain[temp_centre][fract]+temp[temp_centre][fract]);
					temp[temp_centre][fract]+=fabs(grain[temp_centre][fract]+temp[temp_centre][fract]);
					}*/

			}
			if(dist[5][fract]>0.0000){
				if(index[x+1][y+1]==-9999)addGS(x+1,y+1);
				erode4(temp_centre,fract,dist[5][fract],SE);
				
				/*				if(grain[temp_centre][fract]+temp[temp_centre][fract]<0){
				  printf("five %f %f %d %d\n",grain[temp_centre][fract],temp[temp_centre][fract],x,y);
					temp[SE][fract]-=fabs(grain[temp_centre][fract]+temp[temp_centre][fract]);
					temp[temp_centre][fract]+=fabs(grain[temp_centre][fract]+temp[temp_centre][fract]);
					}*/

			}
			if(dist[6][fract]>0.0000){
				if(index[x][y+1]==-9999)addGS(x,y+1);
				erode4(temp_centre,fract,dist[6][fract],S);
				
				/*		if(grain[temp_centre][fract]+temp[temp_centre][fract]<0){
				  	printf("six %f %f %d %d\n",grain[temp_centre][fract],temp[temp_centre][fract],x,y);
					temp[S][fract]-=fabs(grain[temp_centre][fract]+temp[temp_centre][fract]);
					temp[temp_centre][fract]+=fabs(grain[temp_centre][fract]+temp[temp_centre][fract]);
					}*/

			}
			if(dist[7][fract]>0.0000){
				if(index[x-1][y+1]==-9999)addGS(x-1,y+1);
				erode4(temp_centre,fract,dist[7][fract],SW);

		      		/*if(grain[temp_centre][fract]+temp[temp_centre][fract]<0){
				  	printf("seven %f %f %d %d\n",grain[temp_centre][fract],temp[temp_centre][fract],x,y);
					temp[SW][fract]-=fabs(grain[temp_centre][fract]+temp[temp_centre][fract]);
					temp[temp_centre][fract]+=fabs(grain[temp_centre][fract]+temp[temp_centre][fract]);
					}*/

			}
			if(dist[8][fract]>0.0000){
				if(index[x-1][y]==-9999)addGS(x-1,y);
				erode4(temp_centre,fract,dist[8][fract],W);
				
				/*if(grain[temp_centre][fract]+temp[temp_centre][fract]<0){
				  	printf("eight %f %f %d %d\n",grain[temp_centre][fract],temp[temp_centre][fract],x,y);
					temp[W][fract]-=fabs(grain[temp_centre][fract]+temp[temp_centre][fract]);
					temp[temp_centre][fract]+=fabs(grain[temp_centre][fract]+temp[temp_centre][fract]);
					}*/

			}
			
	}

}


/****************************************************************/


void erode4(cent_from,fract,amount,cent_to)
int cent_from;
int fract;
double amount;
int cent_to;
{
  int n;
  double total,sub;
  total=0;

  for(n=0;n<=(TRACERS-1);n++){
    total+=grain[cent_from][fract][n];
  }

  if(total>0){
    for(n=0;n<=(TRACERS-1);n++){
      sub=amount*(grain[cent_from][fract][n]/total)/* *0.999999 */;
      temp[cent_from][fract][n]-=sub;
      temp[cent_to][fract][n]+=sub;
    }
  }
}

/******************************************************************/

float grain_tot(cent,fract)
int cent;
int fract;
{
  int n;
  double total=0;
  for(n=0;n<=(TRACERS-1);n++){
    total+=grain[cent][fract][n];
  }
  return total;
}


float temp_tot(cent,fract)
int cent;
int fract;
{
  int n;
  double total=0;
  for(n=0;n<=(TRACERS-1);n++){
    total+=temp[cent][fract][n];
  }
  return total;
}



float strata_tot(cent,z,fract)
int cent;
int z;
int fract;
{
  int n;
  double total=0;
  for(n=0;n<=(TRACERS-1);n++){
    total+=strata[cent][z][fract][n];
  }
  return total;
}



/************************* revised landslides ***********************/


/***************************** landlsides ***************************/

void slide(scan)
int scan;
{
  int fail_angle=60;
  float diff,main_crit_ht, crit_ht, wet_crit_ht;
  int slide_tot=0;
  int tot,n, x;
  main_crit_ht=tan((fail_angle*(3.141592654/180)))*DX;
  crit_ht=main_crit_ht;
  wet_crit_ht=crit_ht/2;
/*  printf("  Landslide module 1!!! \n");*/


for (scan=1;scan<reach;scan++){

  x=0;

  do
  {
    tot=0;
    for(n=xsection;n>1;n--)
    {
	if(elev[n][scan]>0&&elev[n-1][scan]>0){
	      if(elev[n][scan]<=(bedrock[n][scan]+active))
		crit_ht=10*DX;
       	      else
		crit_ht=main_crit_ht;

  	      if((elev[n][scan]-elev[n-1][scan])>crit_ht)
 	     	{
		diff=(elev[n][scan]-elev[n-1][scan])-crit_ht;
		if((elev[n][scan]-diff)<(bedrock[n][scan]+active))diff=(elev[n][scan]-(bedrock[n][scan]+active));
		slide_tot=slide_tot+diff;
		elev[n][scan]=elev[n][scan]-diff;
		elev[n-1][scan]=elev[n-1][scan]+diff;
		if(index[n][scan]!=-9999||index[n-1][scan]!=-9999)slide_GS(n,scan,diff,n-1,scan);
		tot++;
	      }
	}
    }

    for(n=2;n<xsection;n++)
      {
	if(elev[n][scan]>0&&elev[n-1][scan]>0){
	      if(elev[n-1][scan]<=(bedrock[n-1][scan]+active))
 	       crit_ht=10*DX;
	      else
 	       crit_ht=main_crit_ht;

  	      if((elev[n-1][scan]-elev[n][scan])>crit_ht)
		{
	        diff=(elev[n-1][scan]-elev[n][scan])-crit_ht;
		if((elev[n-1][scan]-diff)<(bedrock[n-1][scan]+active))diff=(elev[n-1][scan]-(bedrock[n-1][scan]+active));
 	        slide_tot=slide_tot+diff;
  	        elev[n-1][scan]=elev[n-1][scan]-diff;
    	        elev[n][scan]=elev[n][scan]+diff;
		if(index[n][scan]!=-9999||index[n-1][scan]!=-9999)slide_GS(n-1,scan,diff,n,scan);
 	        tot++;
	      }
	}
    }

  x++;
  }while(tot>2&&x<200);
}

for (n=1;n<=xsection;n++){

  x=0;

  do
  {
    tot=0;
    for(scan=reach-1;scan>1;scan--)
    {
	if(elev[n][scan]>0&&elev[n][scan-1]>0){
	      if(elev[n][scan]<=(bedrock[n][scan]+active))
		crit_ht=10*DX;
	      else
		crit_ht=main_crit_ht;

	      if((elev[n][scan]-elev[n][scan-1])>crit_ht)
	      {
		diff=(elev[n][scan]-elev[n][scan-1])-crit_ht;
		if((elev[n][scan]-diff)<(bedrock[n][scan]+active))diff=(elev[n][scan]-(bedrock[n][scan]+active));
		slide_tot=slide_tot+diff;
		elev[n][scan]=elev[n][scan]-diff;
		elev[n][scan-1]=elev[n][scan-1]+diff;
		if(index[n][scan-1]!=-9999||index[n][scan]!=-9999)slide_GS(n,scan,diff,n,scan-1);
		tot++;
	      }
	}
    }

    for(scan=2;scan<reach;scan++)
      {
	if(elev[n][scan]>0&&elev[n][scan-1]>0){
	      if(bedrock[n][scan-1]<=(bedrock[n][scan-1]+active))
	        crit_ht=10*DX;
	      else
 	       crit_ht=main_crit_ht;

	      if((elev[n][scan-1]-elev[n][scan])>crit_ht)
		{
	        diff=(elev[n][scan-1]-elev[n][scan])-crit_ht;
		if((elev[n][scan-1]-diff)<(bedrock[n][scan-1]+active))diff=(elev[n][scan-1]-(bedrock[n][scan-1]+active));
	        slide_tot=slide_tot+diff;
	        elev[n][scan-1]=elev[n][scan-1]-diff;
	        elev[n][scan]=elev[n][scan]+diff;
		if(index[n][scan]!=-9999||index[n][scan-1]!=-9999)slide_GS(n,scan-1,diff,n,scan);	      
	        tot++;
 	     }
	}
    }

  x++;
  }while(tot>2&&x<200);
}


}

/********* set grainsizes to init after slides *********/

void slide_GS(x,y,amount,x2,y2)
int x;
int y;
float amount;
int x2;
int y2;
{

  /** Ok, heres how it works, x and y are ones material moved from,
    x2 and y2 are ones material moved to...
    amd amount is the amount shifted. */

	int n,z;
	float total=0;

	if(amount<0)printf("Oh Shit !\n");

	/* if not added to system add in */

	if(index[x][y]==-9999)addGS(x,y);
	if(index[x2][y2]==-9999)addGS(x2,y2);

	/* as these cells are landsliding, get rid of all grass */
	/*  washed away.... except from one that material is
	being moved to, this can keep grass, and let it get buried...  */

	grain[CENTRE][0][0]=0;
	sort_active(x,y);		
	sort_active(x2,y2);
	
	/* then move stuff about ... */

	/* if less than bedrock depth... */

	if(elev[x][y]<=(bedrock[x][y]+active)){
	  /*grain[index[x2][y2]][G_MAX][0]+=amount;*/
	}

	else{
	  z=0;
	  /*if(x>203&&x<430&&y>24&&y<101)z=1;*/

		if(amount>active){
		  grain[index[x2][y2]][1][z]+=(amount-active)*0.05;
		  grain[index[x2][y2]][2][z]+=(amount-active)*0.15;
		  grain[index[x2][y2]][3][z]+=(amount-active)*0.275;
		  grain[index[x2][y2]][4][z]+=(amount-active)*0.25;
	 	  grain[index[x2][y2]][5][z]+=(amount-active)*0.1;
  		  grain[index[x2][y2]][6][z]+=(amount-active)*0.075;
	 	  grain[index[x2][y2]][7][z]+=(amount-active)*0.05;
  		  grain[index[x2][y2]][8][z]+=(amount-active)*0.025;
	 	  grain[index[x2][y2]][9][z]+=(amount-active)*0.025;

	  	  amount=active;
		}
	

		for(n=1;n<=(G_MAX-1);n++){
			if(grain_tot(CENTRE,n)>0.000)total+=grain_tot(CENTRE,n);
		}

		for(n=1;n<=(G_MAX-1);n++){
		  for(z=0;z<=(TRACERS-1);z++){
		    if(total>0){
			grain[index[x2][y2]][n][z]+=amount*(grain[CENTRE][n][z]/total);
			if(grain[CENTRE][n][z]>0.0001)grain[CENTRE][n][z]-=amount*(grain[CENTRE][n][z]/total);
			if(grain[CENTRE][n][z]<0)grain[CENTRE][n][z]=0;
		    }
		  }
		}

	}

	/* then to set active layer to correct depth before erosion, */
	sort_active(x,y);
	sort_active(x2,y2);

}


/************** loads dtm data  ****************/

void load_data()
{
  int x,y;

  FILE *ifp;
  /*   ifp=fopen("blank9.txt","r");*/
  ifp=fopen("star3dem3.dat","r");
  /*
  for(y=1;y<=2000;y++){
    fscanf(ifp,"\n");
  }*/
  for(y=1;y<=reach;y++){
    for(x=1;x<=xsection;x++){
      fscanf(ifp,"%f ",&elev[x][y]);
    }
    fscanf(ifp,"\n");
  }
  printf("Poo !!\n");
  fclose(ifp);
  printf("poo !!\n");
}

/****** saves data  ******************/

void save_data(cycle,xmin,xmax,ymin,ymax)
float cycle;
int xmin;
int xmax;
int ymin;
int ymax;
{
  char filename[15];
  int x,y;
  FILE *ofp;
  sprintf(filename,"E%f",cycle);
  ofp=fopen(filename,"w");

  fprintf(ofp,"ncols         %d\n",(xmax-xmin)+1);
  fprintf(ofp,"nrows         %d\n",(ymax-ymin)+1);
  fprintf(ofp,"xllcorner     0.0\n");
  fprintf(ofp,"yllcorner     0.0\n");
  fprintf(ofp,"cellsize      1.0\n");
  fprintf(ofp,"NODATA_value  -9999\n");

  for (y=ymin;y<=ymax;y++){
    for(x=xmin;x<=xmax;x++){

	fprintf(ofp,"%f ",elev[x][y]);

/*      fprintf(ofp,"%f ",(water_depth[x][y]+elev[x][y]));*/
    }
    fprintf(ofp,"\n");
  }
  fclose(ofp);
}


void save_data2(cycle)
float cycle;
{
  char filename[15];
  int x,y,inc,z;
  float tot, tot_max;
  FILE *ofp;
  sprintf(filename,"update.dat");
  ofp=fopen(filename,"w");

  fprintf(ofp,"ncols         %d\n",xsection);
  fprintf(ofp,"nrows         %d\n",reach);
  fprintf(ofp,"xllcorner     0\n");
  fprintf(ofp,"yllcorner     0\n");
  fprintf(ofp,"cellsize      %d\n",DX);
  fprintf(ofp,"NODATA_value  -9999\n");

  for (y=1;y<=reach;y++){
    for(x=1;x<=xsection;x++){
      if(index[x][y]!=-9999){
	tot=0;
	tot_max=0;
	for(inc=0;inc<=G_MAX;inc++){
	  tot+=grain[CENTRE][inc][1];
	}
	if(tot>tot_max)tot_max=tot;
	for(z=0;z<=9;z++){
	  tot=0;
	  for(inc=0;inc<=(G_MAX-2);inc++){
	    tot+=strata[CENTRE][z][inc][1];
	  }
	  if(tot>tot_max)tot_max=tot;
	}
	fprintf(ofp,"%f ",(tot_max/0.2)*100);
      }
      else{
	fprintf(ofp,"-9999.0 ");
      }
    }
    fprintf(ofp,"\n");
  }
  fclose(ofp);


}

/*** save data3.dat **/

void save_data3(cycle)
float cycle;
{
  char filename[15];
  int x,y,inc,z;
  float tot, tot_max;
  FILE *ofp;
  sprintf(filename,"flowdirin.dat");
  ofp=fopen(filename,"w");

  fprintf(ofp,"ncols         %d\n",xsection);
  fprintf(ofp,"nrows         %d\n",reach);
  fprintf(ofp,"xllcorner     0\n");
  fprintf(ofp,"yllcorner     0\n");
  fprintf(ofp,"cellsize      %d\n",DX);
  fprintf(ofp,"NODATA_value  -9999\n");

  for (y=1;y<=reach;y++){
    for(x=1;x<=xsection;x++){
      if(flow_dir[x][y][0]>0){
	fprintf(ofp,"%d.0 ",flow_dir[x][y][0]);
      }
      else{
	fprintf(ofp,"-9999.0 ");
      }
    }
    fprintf(ofp,"\n");
  }
  fclose(ofp);

  sprintf(filename,"flowdirout.dat");
  ofp=fopen(filename,"w");

  fprintf(ofp,"ncols         %d\n",xsection);
  fprintf(ofp,"nrows         %d\n",reach);
  fprintf(ofp,"xllcorner     0\n");
  fprintf(ofp,"yllcorner     0\n");
  fprintf(ofp,"cellsize      %d\n",DX);
  fprintf(ofp,"NODATA_value  -9999\n");

  for (y=1;y<=reach;y++){
    for(x=1;x<=xsection;x++){
      if(flow_dir[x][y][1]>0){
	fprintf(ofp,"%d.0 ",flow_dir[x][y][1]);
      }
      else{
	fprintf(ofp,"-9999.0 ");
      }
    }
    fprintf(ofp,"\n");
  }
  fclose(ofp);


  sprintf(filename,"elevdiff.dat");
  ofp=fopen(filename,"w");

  fprintf(ofp,"ncols         %d\n",xsection);
  fprintf(ofp,"nrows         %d\n",reach);
  fprintf(ofp,"xllcorner     0\n");
  fprintf(ofp,"yllcorner     0\n");
  fprintf(ofp,"cellsize      %d\n",DX);
  fprintf(ofp,"NODATA_value  -9999\n");

  for (y=1;y<=reach;y++){
    for(x=1;x<=xsection;x++){

	fprintf(ofp,"%f ",(init_elevs[x][y]-elev[x][y]));
	/*init_elevs[x][y]=elev[x][y];*/

      
    }
    fprintf(ofp,"\n");
  }
  fclose(ofp);



  sprintf(filename,"waterdepth.dat");
  ofp=fopen(filename,"w");

  fprintf(ofp,"ncols         %d\n",xsection);
  fprintf(ofp,"nrows         %d\n",reach);
  fprintf(ofp,"xllcorner     0\n");
  fprintf(ofp,"yllcorner     0\n");
  fprintf(ofp,"cellsize      %d\n",DX);
  fprintf(ofp,"NODATA_value  -9999\n");

  for (y=1;y<=reach;y++){
    for(x=1;x<=xsection;x++){

      if(water_depth[x][y]>0){
	fprintf(ofp,"%f ",water_depth[x][y]);
	/*init_elevs[x][y]=elev[x][y];*/
      }
      else{
	fprintf(ofp,"-9999.0 ");
      }
	
      
    }
    fprintf(ofp,"\n");
  }
  fclose(ofp);


  sprintf(filename,"grain.dat");
  ofp=fopen(filename,"w");

  fprintf(ofp,"ncols         %d\n",xsection);
  fprintf(ofp,"nrows         %d\n",reach);
  fprintf(ofp,"xllcorner     0\n");
  fprintf(ofp,"yllcorner     0\n");
  fprintf(ofp,"cellsize      %d\n",DX);
  fprintf(ofp,"NODATA_value  -9999\n");

  for (y=1;y<=reach;y++){
    for(x=1;x<=xsection;x++){
      if(index[x][y]!=-9999){
	tot=0;
	tot_max=0;

	tot=((grain[CENTRE][1][0]*1)+(grain[CENTRE][2][0]*2)+(grain[CENTRE][3][0]*4)+(grain[CENTRE][4][0]*8)+(grain[CENTRE][5][0]*16)+(grain[CENTRE][6][0]*32)+(grain[CENTRE][7][0]*64)+(grain[CENTRE][8][0]*128));
	fprintf(ofp,"%f ",tot);
      }
      else{
	fprintf(ofp,"-9999.0 ");
      }
    }
    fprintf(ofp,"\n");
  }
  fclose(ofp);



}


/************* save all data properly ********/

void save_data50(cycle_temp)
int cycle_temp;
{
  char filename[25];
  int x,y,inc,z;
  float tot, tot_max;
  FILE *ofp;
  sprintf(filename,"tracer.dat.%d",cycle_temp);
  ofp=fopen(filename,"w");

  fprintf(ofp,"ncols         %d\n",xsection);
  fprintf(ofp,"nrows         %d\n",reach);
  fprintf(ofp,"xllcorner     0\n");
  fprintf(ofp,"yllcorner     0\n");
  fprintf(ofp,"cellsize      %d\n",DX);
  fprintf(ofp,"NODATA_value  -9999\n");

  for (y=1;y<=reach;y++){
    for(x=1;x<=xsection;x++){
      fprintf(ofp,"%f ",elev[x][y]);
    }
    fprintf(ofp,"\n");
  }
  fclose(ofp);



  sprintf(filename,"tracerg.dat.%d",cycle_temp);
  ofp=fopen(filename,"w");

  for (y=1;y<=reach;y++){
    for(x=1;x<=xsection;x++){
      if(index[x][y]!=-9999){
	fprintf(ofp,"%d %d %d ",x,y,index[x][y]);
	for(inc=0;inc<=G_MAX;inc++){
	  fprintf(ofp,"%f ",grain[CENTRE][inc][0]);
	}
	
	for(z=0;z<=9;z++){
	  for(inc=0;inc<=(G_MAX-2);inc++){
	    fprintf(ofp,"%f ",strata[CENTRE][z][inc][0]);
	  }
	}
	
	fprintf(ofp,"\n");
      }
    }
  }
  
  fclose(ofp); 
  sprintf(filename,"tracer-cont.dat.%d",cycle_temp);
  ofp=fopen(filename,"w");

  fprintf(ofp,"ncols         %d\n",xsection);
  fprintf(ofp,"nrows         %d\n",reach);
  fprintf(ofp,"xllcorner     0\n");
  fprintf(ofp,"yllcorner     0\n");
  fprintf(ofp,"cellsize      %d\n",DX);
  fprintf(ofp,"NODATA_value  -9999\n");

  for (y=1;y<=reach;y++){
    for(x=1;x<=xsection;x++){
      if(index[x][y]!=-9999){
	tot=0;
	tot_max=0;
	for(inc=0;inc<=G_MAX;inc++){
	  tot+=grain[CENTRE][inc][1];
	}
	if(tot>tot_max)tot_max=tot;
	for(z=0;z<=9;z++){
	  tot=0;
	  for(inc=0;inc<=(G_MAX-2);inc++){
	    tot+=strata[CENTRE][z][inc][1];
	  }
	  if(tot>tot_max)tot_max=tot;
	}
	fprintf(ofp,"%f ",(tot_max/0.2)*100);
      }
      else{
	fprintf(ofp,"-9999.0 ");
      }
    }
    fprintf(ofp,"\n");
  }
  fclose(ofp);


  sprintf(filename,"tracerg-cont.dat.%d",cycle_temp);
  ofp=fopen(filename,"w");

  for (y=1;y<=reach;y++){
    for(x=1;x<=xsection;x++){
      if(index[x][y]!=-9999){
	fprintf(ofp,"%d %d %d ",x,y,index[x][y]);
	for(inc=0;inc<=G_MAX;inc++){
	  fprintf(ofp,"%f ",grain[CENTRE][inc][1]);
	}
	
	for(z=0;z<=9;z++){
	  for(inc=0;inc<=(G_MAX-2);inc++){
	    fprintf(ofp,"%f ",strata[CENTRE][z][inc][1]);
	  }
	}
	
	fprintf(ofp,"\n");
      }
    }
  }
  
  fclose(ofp); 
}


/******** saves all data for next run. *****/


void save_all_data(cycle)
float cycle;
{
	
 	int x,y,n,z,q, inc;
	float buffer,subtot;
	FILE *ofp;

 	ofp=fopen("tracer.dat","w");

	fprintf(ofp,"%f\n",cycle);

	for (y=1;y<=reach;y++){
		for(x=1;x<=xsection;x++){
			fprintf(ofp,"%f ",elev[x][y]);
		 
		  
 	   	}
  	  	fprintf(ofp,"\n");
	}
	fclose(ofp);

	ofp=fopen("tracerg.dat","w");

        for (y=1;y<=reach;y++){
	        for(x=1;x<=xsection;x++){
		  if(index[x][y]!=-9999){
		    fprintf(ofp,"%d %d %d ",x,y,index[x][y]);
		    for(inc=0;inc<=G_MAX;inc++){
                        fprintf(ofp,"%f ",grain[CENTRE][inc][0]);
		    }

		    for(z=0;z<=9;z++){
			for(inc=0;inc<=(G_MAX-2);inc++){
                             fprintf(ofp,"%f ",strata[CENTRE][z][inc][0]);
			}
		    }

		    fprintf(ofp,"\n");
		  }
	        }
        }

        fclose(ofp);
 

}


/*********** loads all data for runb **********/



void load_all_data()
{
	
	float buffer,lowest,temp,temp1,temp2;
 	int x,y,n, flag,inc,z,c;
	FILE *ifp;

 	ifp=fopen("hourly_rain_data.dat","r");

	inc=0;
	while(!feof(ifp)){

		inc++;
			
		fscanf(ifp,"%f ",&hourly_rain_data[inc]);
 	  	fscanf(ifp,"\n");
		/*printf("%f\n",hourly_rain_data[inc]);*/

		hourly_rain_data[inc]=hourly_rain_data[inc]*1;
	
	}
	
	fclose(ifp);

	printf("done rain data\n");

 	ifp=fopen("climate4.dat","r");
	
	while(!feof(ifp)){
       
		fscanf(ifp,"%d %f %f\n ",&x,&temp1,&temp2);
		climate_data[x][1]=temp1;
		climate_data[x][2]=temp2;
	}


	fclose(ifp);
	printf("done climate data \n");

 	ifp=fopen("minesites2.txt","r");
	inc=0;
	while(!feof(ifp)){
   		
		fscanf(ifp,"%d ",&x);
		fscanf(ifp,"%d ",&y);
 	  	fscanf(ifp,"\n");
		minesites[inc][0]=x;
		minesites[inc][1]=y;
		/*printf("%d %d\n",minesites[inc][0],minesites[inc][1]);*/
		inc++;
	}
	
	fclose(ifp);

	printf("done mine sites\n");


	ifp=fopen("test4.dat","r");

	for (y=1;y<=reach;y++){
		lowest=1000;
		for(x=1;x<=xsection;x++){

			fscanf(ifp,"%f ",&temp);
			if(temp>0)bedrock[x][y]=temp-10;
	   	}
  	  	fscanf(ifp,"\n");

	}
	fclose(ifp);

	printf("done DEM\n");


 	ifp=fopen("surfacebedrock.txt","r");
	inc=0;
	while(!feof(ifp)){
   		
		fscanf(ifp,"%d ",&x);
		fscanf(ifp,"%d ",&y);
 	  	fscanf(ifp,"\n");
		bedrock[x][y]+=10;
		/*printf("%d %d\n",minesites[inc][0],minesites[inc][1]);*/
		inc++;
	}
	
	fclose(ifp);

	printf("done surface bedrock\n");


 	ifp=fopen("tracerg.dat.init","r");
	
	grain_array_tot=0;
	while(!feof(ifp)){
		grain_array_tot++;
		fscanf(ifp,"%d %d ",&x,&y);
		c=0;
		/*if(x>203&&x<255&&y>24&&y<101)c=1;*/
		

			fscanf(ifp,"%d ",&buffer);
			index[x][y]=grain_array_tot;
			for(inc=0;inc<=G_MAX;inc++){
				fscanf(ifp,"%f ",&buffer);
				grain[grain_array_tot][inc][c]=buffer;
			
			}

        	    for(z=0;z<=9;z++){
			for(inc=0;inc<=(G_MAX-2);inc++){
                             fscanf(ifp,"%f ",&buffer);
				strata[CENTRE][z][inc][c]=buffer;
			}
		    }

 		fscanf(ifp,"\n");
	}


	fclose(ifp);
	printf("done grain data \n");


	ifp=fopen("tracer.dat.init","r");

	fscanf(ifp,"%f\n",&buffer);
	cycle=buffer;

	if(cycle>5200000)cycle=10;/* RMEEBRER TO TAKE OUT !! */

	for (y=1;y<=reach;y++){
		lowest=1000;
		for(x=1;x<=xsection;x++){

			fscanf(ifp,"%f ",&buffer);
		    
			elev[x][y]=(buffer);
			init_elevs[x][y]=elev[x][y];
			/*if(elev[x][y]>0)bedrock[x][y]=elev[x][y]-1;*/
			/*	if(bedrock_depth[y]>bedrock[x][y])bedrock[x][y]=bedrock_depth[y];*/
 			/*if(elev[x][y]<lowest&&elev[x][y]>1){
				if(x>135||y>1905)lowest=elev[x][y];
			}*/
	   	}
  	  	fscanf(ifp,"\n");
	/*	printf("%f\n",lowest-1);*/

	}
	fclose(ifp);

/*
	for(y=1;y<=1000;y++){
		printf("\n");
	}
*/
	printf("done elevs \n");       


}














